"""
Module to run OptB88vdW based High-throughput calculations
"""
from __future__ import division, unicode_literals, print_function
import os,socket,shutil
from monty.json import MontyEncoder, MontyDecoder
from custodian.vasp.jobs import VaspJob
from pymatgen.io.vasp import VaspInput, Vasprun
from custodian.vasp.jobs import VaspJob
from pymatgen.io.vasp.outputs import Oszicar
from subprocess import Popen, PIPE
from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
import sys,shutil,glob,codecs
from pymatgen.io.ase import AseAtomsAdaptor
from pymatgen.core.surface import  Slab, SlabGenerator, generate_all_slabs,get_symmetrically_distinct_miller_indices
from custodian.vasp.handlers import VaspErrorHandler, UnconvergedErrorHandler, \
    MeshSymmetryErrorHandler, NonConvergingErrorHandler, PotimErrorHandler
from pymatgen.symmetry.bandstructure import HighSymmKpath
import json,yaml
from numpy import linalg as LA
import time
from collections import OrderedDict
from jarvis.lammps.jlammps import vac_antisite_def_struct_gen,surfer
#from jarvis.lammps.Surf_Def import vac_antisite_def_struct_gen,surfer
from pymatgen.matproj.rest import MPRester
import subprocess
from pymatgen.core.structure import Structure
from pymatgen.io.vasp.inputs import Incar, Poscar, VaspInput
from pymatgen.io.vasp.inputs import Potcar, Kpoints
#from pymatgen.io.vasp.sets import MPVaspInputSet, MPNonSCFVaspInputSet
from numpy import matrix
import numpy as np
from pymatgen.io.ase import AseAtomsAdaptor
import operator
from monty.json import MontyEncoder, MontyDecoder
from monty.serialization import loadfn, dumpfn

main_exe=os.environ['vasp_bulk_exe']
surf_exe= os.environ['vasp_surf_exe']
nw_exe= os.environ['vasp_nw_exe']
soc_exe=os.environ['vasp_soc_exe']
pot_yaml=os.environ['vasp_pot_yaml']
vdw_dat=os.environ['vasp_vdw_dat']
json_dat=os.environ['mp_json']




      
def check_polar(file):
    up=0
    dn=0
    coords = np.array(file.frac_coords)
    z_max = max(coords[:, 2])
    z_min=  min(coords[:, 2])
    for site in file:
        if (site.frac_coords[2]==z_max) :
           up=up+site.specie.number
        if (site.frac_coords[2]==z_min) :
           dn=dn+site.specie.number
    polar=False
    if(up !=dn):
       print ('polar')
       polar=True
    if(up == dn):
       print ('Non-polar')
       polar=False
    return polar


def get_lowest_en_from_mp(formula, MAPI_KEY="", all_structs=False):
    """
    fetches the structure corresponding to the given formula
    from the materialsproject database.
    
    Note: Get the api key from materialsproject website. The one used
    here is nolonger valid.
    
    Note: for the given formula there are many structures available, 
    this function returns the one with the lowest energy above the hull
    unless all_structs is set to True
    """
    if not MAPI_KEY:
        MAPI_KEY = os.environ.get("MAPI_KEY", "")
        if not MAPI_KEY:
            print('API key not provided')
            print('get API KEY from materialsproject and set it to the MAPI_KEY environment variable. aborting ... ')
            sys.exit()
    with MPRester(MAPI_KEY) as m:
        data = m.get_data(formula)
        structures = []
        x = {}
        print("\nnumber of structures matching the chemical formula {0} = {1}".format(formula, len(data)))
        print("The one with the the lowest energy above the hull is returned, unless all_structs is set to True")
        for d in data:
            mpid = str(d['material_id'])
            x[mpid] = d['e_above_hull']
            if all_structs:
                structure = m.get_structure_by_material_id(mpid)
                structures.append(structure)
        else:
            mineah_key = sorted(x.items(), key=operator.itemgetter(1))[0][0]
            print("The id of the material corresponding to the lowest energy above the hull = {0}".format(mineah_key))
            if mineah_key:
                with MPRester(MAPI_KEY) as m:
                    data = m.get_data(mineah_key)
                    x = {}
                    for d in data:
                         x['energy_per_atom'] = str(d['energy_per_atom'])
                         enp= x['energy_per_atom']
                #return m.get_structure_by_material_id(mineah_key)
                return enp
            else:
                return None

def sum_chem_pot(strt=None):
    sum=0
    symb=strt.symbol_set
    for el in symb:
        enp=get_lowest_en_from_mp(el)
        sum=float(sum)+float(enp)
    return sum

def run_job(mat=None,incar=None,kpoints=None,jobname='',copy_file=[]):    
    hostname=str(socket.gethostname()) 
    poscar_list=[(mat)]
    nprocs = 16
    nnodes = 8
    mem='1500'
    cwd=str(os.getcwd())
    job_name=str(mat.comment)
    walltime = '148:12:00'
    job_dir=str(jobname)
    run_file = str(os.getcwd())+str('/')+str(jobname)+str('.json')
    run_dir = str(os.getcwd())+str('/')+str(jobname)
    if mat.comment.startswith('Surf'):
       [a,b,c]=kpoints.kpts[0]
       kpoints.kpts=[[a,b,1]]
       
       try:
           pol=check_polar(mat.structure)
           if pol==True:
                ase_atoms = AseAtomsAdaptor().get_atoms(mat.structure)
                COM=ase_atoms.get_center_of_mass(scaled=True)
                print ("COM=",COM)
                incar.update({"LDIPOL": '.TRUE.',"IDIPOL":3,"ISYM": 0,"DIPOL":str(COM[0])+str(" ")+str(COM[2])+str(" ")+str(COM[2])})
                print ("Polar surface encountered in run_job",mat.comment)
       except:
            pass
    wait=False
    json_file=str(jobname)+str('.json')
    print ('json should be=',json_file,run_file,os.getcwd())
    if hostname!='ratan.hpc.nist.gov':
       print ('raritan.hpc.nist.gov 1 ')
       try:  
         if os.path.isfile(run_file):
            d=loadfn(run_file,cls=MontyDecoder)
            f_energy=d[0]['final_energy']      
            contcar=run_file.split('.json')[0]+str("/CONTCAR") 
            wait=True 
            print ('raritan.hpc.nist.gov 2 ',f_energy)
       except:
           pass  
    if wait==False:
            print ("I AM HERE 2")
            with open(pot_yaml, 'r') as f:
                 doc = yaml.load(f)
                 pots=doc['POTCAR']
            new_symb=[]
            for el in mat.site_symbols:
               new_symb.append(pots[el])
            potcar = Potcar(symbols=new_symb,functional="PBE")
            if not os.path.exists(run_dir):
                                                   os.makedirs(jobname)
                                                   os.chdir(jobname)
                                                   incar.write_file("INCAR")
                                                   potcar.write_file("POTCAR")
                                                   kpoints.write_file("KPOINTS")
                                                   mat.write_file("POSCAR")
                       
            
                                                   for i in copy_file:
                                                      print ('copying',i)
                                                      shutil.copy2(i,'./')
                                                   #f=open('job.out','w')
                                              
                                                   cmd=str('python  first_cust.py >ouuu')+'\n' 
                                                   cust_file=open("first_cust.py","w")
                                                   cline=str('from pymatgen.io.vasp.inputs import Incar, Poscar, VaspInput,Potcar, Kpoints')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('import os,shutil')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('from custodian.vasp.jobs import VaspJob')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('from custodian.vasp.handlers import VaspErrorHandler, UnconvergedErrorHandler,MeshSymmetryErrorHandler, NonConvergingErrorHandler, PotimErrorHandler')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('from custodian.vasp.validators import VasprunXMLValidator')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('from custodian.custodian import Custodian')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('inc=Incar.from_file("INCAR")')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('pot=Potcar.from_file("POTCAR")')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('pos=Poscar.from_file("POSCAR")')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('kp=Kpoints.from_file("KPOINTS")')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str("shutil.copy2('")+vdw_dat+str("','./')")+'\n'
                                                   cust_file.write(cline)
                                                   cline=str('vinput = VaspInput.from_directory(".")')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str("job=VaspJob(['mpirun', '")+str(main_exe)+str("'], final=False, backup=False)")+'\n' 
                                                   if mat.comment.startswith('Surf'):
                                                      cline=str("job=VaspJob(['mpirun',  '")+str(surf_exe)+str("'], final=False, backup=False)")+'\n' 
                                                   if 'SOC' in jobname:
                                                      cline=str("job=VaspJob(['mpirun',  '")+str(soc_exe)+str("'], final=False, backup=False)")+'\n' 
                                                   
                                                   cust_file.write(cline)
                                                   cline=str('handlers = [VaspErrorHandler(), MeshSymmetryErrorHandler(),UnconvergedErrorHandler(), NonConvergingErrorHandler(),PotimErrorHandler()]')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('validators = [VasprunXMLValidator()]')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('c = Custodian(handlers, [job],max_errors=5,validators=validators)')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('c.run()')+'\n' 
                                                   cust_file.write(cline)
                                                   cust_file.close()
                                                   print ("I AM HERE 2")
                                                   os.system(cmd)
                                                   try:
                                                      wait=Vasprun("vasprun.xml").converged
                                                   except:
                                                           pass 
                                                   print ("I AM HERE 3",os.getcwd(),wait)


            else :
                  os.chdir(jobname)
                  wait=False
                  if os.path.isfile('vasprun.xml'):
                     try:
                         wait=Vasprun("vasprun.xml").converged
                     except:
                          pass
                  print ("wait os=",wait)
                  if wait==False:
                                                   incar.write_file("INCAR")
                                                   potcar.write_file("POTCAR")
                                                   kpoints.write_file("KPOINTS")
                                                   mat.write_file("POSCAR")
                                                   try:
                                                      print ('FOUND OLD CONTCAR in', os.getcwd())
                                                      old_contcar=Poscar.from_file('CONTCAR')
                                                      old_contcar.write_file('POSCAR')
                                                   except:
                                                        pass           
                                                   for i in copy_file:
                                                      print ('copying',i)
                                                      shutil.copy2(i,'./')
            
                                              
                                                   cmd=str('python  first_cust.py >ouuu')+'\n' 
                                                   cust_file=open("first_cust.py","w")
                                                   cline=str('from pymatgen.io.vasp.inputs import Incar, Poscar, VaspInput,Potcar, Kpoints')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('import os,shutil')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('from custodian.vasp.jobs import VaspJob')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('from custodian.vasp.handlers import VaspErrorHandler, UnconvergedErrorHandler,MeshSymmetryErrorHandler, NonConvergingErrorHandler, PotimErrorHandler')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('from custodian.vasp.validators import VasprunXMLValidator')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('from custodian.custodian import Custodian')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('inc=Incar.from_file("INCAR")')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('pot=Potcar.from_file("POTCAR")')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('pos=Poscar.from_file("POSCAR")')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('kp=Kpoints.from_file("KPOINTS")')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str("shutil.copy2('")+vdw_dat+str("','./')")+'\n'
                                                   cust_file.write(cline)
                                                   cline=str('vinput = VaspInput.from_directory(".")')+'\n' 
                                                   cust_file.write(cline)


                                                   cline=str("job=VaspJob(['mpirun', '")+str(main_exe)+str("'], final=False, backup=False)")+'\n' 
                                                   if mat.comment.startswith('Surf'):
                                                      cline=str("job=VaspJob(['mpirun',  '")+str(surf_exe)+str("'], final=False, backup=False)")+'\n' 
                                                   if 'SOC' in jobname:
                                                      cline=str("job=VaspJob(['mpirun',  '")+str(soc_exe)+str("'], final=False, backup=False)")+'\n' 

                                                   
                                                   cust_file.write(cline)
                                                   cline=str('handlers = [VaspErrorHandler(), MeshSymmetryErrorHandler(),UnconvergedErrorHandler(), NonConvergingErrorHandler(),PotimErrorHandler()]')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('validators = [VasprunXMLValidator()]')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('c = Custodian(handlers, [job],max_errors=5,validators=validators)')+'\n' 
                                                   cust_file.write(cline)
                                                   cline=str('c.run()')+'\n' 
                                                   cust_file.write(cline)
                                                   cust_file.close()
                                                   os.system(cmd)





            contcar=str(os.getcwd())+str('/')+str('CONTCAR')
            oszicar = Oszicar("OSZICAR")
            final_str=Structure.from_file(contcar)
            f_energy=float(oszicar.final_energy)
            natoms=final_str.composition.num_atoms
            enp=float(oszicar.final_energy)/float(final_str.composition.num_atoms)
            os.chdir('../')
            data_cal=[]
            data_cal.append({'jobname':jobname,'poscar_initial':mat.as_dict(),'poscar_final':final_str.as_dict(),'incar':incar.as_dict(),'kpoints':kpoints.as_dict(),'final_energy':float(oszicar.final_energy),'contcar':final_str.as_dict()})
            json_file=str(jobname)+str('.json')
            f_json=open(json_file,'w')
            f_json.write(json.dumps(data_cal,indent=4,cls=MontyEncoder))
            f_json.close()
    print ('I AM HEREEEE',contcar)
    return f_energy,contcar



##########################################################
def check_errors(logfile='vasp.out',timeout=18000):
    errors = []
    #print ("going here 12")
    try:
      run=Vasprun("vasprun.xml")
      if run.converged_electronic == False:
          errors.append("unconverged_electronic")
      if run.converged_ionic == False:
          errors.append("unconverged")
    except:
        pass
    #f=open("OUTCAR",'r')
    try:
       oszicar = Oszicar("OSZICAR")
       ionic_steps=len(oszicar.ionic_steps)
       with open("OUTCAR") as f:
             for line in f:

                 if "NSW" in line:
                     try:
                         #nbands = int(d[-1].strip())
                         nsw= int(str(line.split("=")[1]).split("number of steps")[0])
                         if nsw >1 and ionic_steps==nsw:
                            errors.append("unconverged") 
                     except:
                        pass

                 if "NELM" in line:
                     try:
                         #nbands = int(d[-1].strip())
                         #nsw= int(str(line.split("=")[1]).split("number of steps")[0])
                         nelm= int(str(line.split("=")[1]).split(";")[0])
                         electronic_steps=len(os.electronic_steps[-1])
                         if electronic_steps <nelm:
                             print("Electronically converged")
                         else:
                            errors.append("unconverged_electronic") 
            
                     except:
                        pass

 

    except:
          
         pass
              

    with open(logfile, "r") as f:
        for line in f:
            l = line.strip()

            if "WARNING: Sub-Space-Matrix is not hermitian in" in  line:
                     err="subspacematrix"
                     if err not  in errors:
                        errors.append(err)
            if "Tetrahedron method fails for NKPT<" in  line:
                     err="tet"
                     if err not  in errors:
                        errors.append(err)
            if "Fatal error detecting k-mesh" in  line:
                     err="tet"
                     if err not  in errors:
                        errors.append(err)
            if "Fatal error: unable to match k-point" in  line:
                     err="tet"
                     if err not  in errors:
                        errors.append(err)
            if "Routine TETIRR needs special values" in  line:
                     err="tet"
                     if err not  in errors:
                        errors.append(err)
            if "Tetrahedron method fails for NKPT<" in  line:
                     err="tet"
                     if err not  in errors:
                        errors.append(err)
            if "inverse of rotation matrix was not found (increase" in  line:
                     err="inv_rot_ma"
                     if err not  in errors:
                        errors.append(err)
            if "SYMPREC" in  line:
                     err="inv_rot_ma"
                     if err not  in errors:
                        errors.append(err)
            if "Routine TETIRR needs special values" in  line:
                     err="tetirr"
                     if err not  in errors:
                        errors.append(err)
            if "Could not get correct shift" in  line:
                     err="incorrect_shift"
                     if err not  in errors:
                        errors.append(err)
            if "REAL_OPTLAY: internal error" in  line:
                     err="real_optlay"
                     if err not  in errors:
                        errors.append(err)
            if "REAL_OPT: internal ERROR" in  line:
                     err="real_optlay"
                     if err not  in errors:
                        errors.append(err)
            if "ERROR RSPHER" in  line:
                     err="rspher"
                     if err not  in errors:
                        errors.append(err)
            if "DENTET" in  line:
                     err="dentet"
                     if err not  in errors:
                        errors.append(err)
            if "TOO FEW BAND" in  line:
                     err="too_few_bands"
                     if err not  in errors:
                        errors.append(err)
            if "ERROR: the triple product of the basis vectors" in  line:
                     err="triple_product"
                     if err not  in errors:
                        errors.append(err)
            if "Found some non-integer element in rotation matrix" in  line:
                     err="rot_matrix"
                     if err not  in errors:
                        errors.append(err)
            if "BRIONS problems: POTIM should be increased" in  line:
                     err="brions"
                     if err not  in errors:
                        errors.append(err)
            if "internal error in subroutine PRICEL" in  line:
                     err="pricel"
                     if err not  in errors:
                        errors.append(err)
            if "LAPACK: Routine ZPOTRF failed" in  line:
                     err="zpotrf"
                     if err not  in errors:
                        errors.append(err)
            if "One of the lattice vectors is very long (>50 A), but AMIN" in  line:
                     err="amin"
                     if err not  in errors:
                        errors.append(err)
            if "ZBRENT: fatal internal in" in  line:
                     err="zbrent"
                     if err not  in errors:
                        errors.append(err)
            if "ZBRENT: fatal error in bracketing" in  line:
                     err="zrbent"
                     if err not  in errors:
                        errors.append(err)

            if "ERROR in subspace rotation PSSYEVX" in  line:
                     err="pssyevx"
                     if err not  in errors:
                        errors.append(err)
            if "WARNING in EDDRMM: call to ZHEGV failed" in  line:
                     err="eddrmm"
                     if err not  in errors:
                        errors.append(err)
            if "Error EDDDAV: Call to ZHEGV failed" in  line:
                     err="edddav"
                     if err not  in errors:
                        errors.append(err)
            if "Your FFT grids (NGX,NGY,NGZ) are not sufficient" in  line:
                     err="aliasing_incar"
                     if err not  in errors:
                        errors.append(err)


#    with open(logfile, "r") as f:
#        for line in f:
#            l = line.strip()
#            for err, msgs in VaspErrorHandler.error_msgs.items():
#                for msg in msgs:
#                    if l.find(msg) != -1:
#                        # this checks if we want to run a charged
#                        # computation (e.g., defects) if yes we don't
#                        # want to kill it because there is a change in e-
#                        # density (brmix error)
#                        if err == "brmix" and 'NELECT' in incar:
#                            continue
                        errors.append(err)
    #UNCONVERGED
    st = os.stat('vasp.out')
    if time.time() - st.st_mtime > timeout:
           errors.append('Frozenjob')

    return set(errors)

def check_errorss(logfile='log'):
    errors = []
    with open(logfile, "r") as f:
        for line in f:
            l = line.strip()
            for err, msgs in VaspErrorHandler.error_msgs.items():
                for msg in msgs:
                    if l.find(msg) != -1:
                        # this checks if we want to run a charged
                        # computation (e.g., defects) if yes we don't
                        # want to kill it because there is a change in e-
                        # density (brmix error)
                        if err == "brmix" and 'NELECT' in incar:
                            continue
                        errors.append(err)
    return set(errors)
def check_error(logfile='log'):
    errors = set()
    with open(logfile, "r") as f:
        for line in f:
            l = line.strip()
            for err, msgs in VaspErrorHandler.error_msgs.items():
                for msg in msgs:
                    if l.find(msg) != -1:
                        # this checks if we want to run a charged
                        # computation (e.g., defects) if yes we don't
                        # want to kill it because there is a change in e-
                        # density (brmix error)
                        if err == "brmix" and 'NELECT' in incar:
                            continue
                        errors.add(err)
    return len(errors) > 0


#b1=LA.norm(np.array(mat.lattice.reciprocal_lattice_crystallographic.matrix[0]))
#b2=LA.norm(np.array(mat.lattice.reciprocal_lattice_crystallographic.matrix[1]))
#b3=LA.norm(np.array(mat.lattice.reciprocal_lattice_crystallographic.matrix[2]))
#print b1,b2,b3

#length=20
#n1=max(1,length*b1+0.5)
#n2=max(1,length*b2+0.5)
#n3=max(1,length*b3+0.5)
#print n1,n2,n3
def Auto_Kpoints(mat=None,length=20):
    #kp_file=open("KPOINTS","w")
    #line=str(mat.comment)+'\n'
    #kp_file.write(line)
    #line=str(0)+'\n'
    #kp_file.write(line)
    #line=str("Auto")+'\n'
    #kp_file.write(line)
    #line=str(length)+'\n'
    #kp_file.write(line)
    #kp_file.close()
    #pym_kp=Kpoints.from_file("KPOINTS")

    b1=LA.norm(np.array(mat.structure.lattice.reciprocal_lattice_crystallographic.matrix[0]))
    b2=LA.norm(np.array(mat.structure.lattice.reciprocal_lattice_crystallographic.matrix[1]))
    b3=LA.norm(np.array(mat.structure.lattice.reciprocal_lattice_crystallographic.matrix[2]))

    n1=int(max(1,length*b1+0.5))
    n2=int(max(1,length*b2+0.5))
    n3=int(max(1,length*b3+0.5))
    kpp=Kpoints.gamma_automatic(kpts=(n1,n2,n3))
    return kpp

### kpp=Kpoints.gamma_automatic(kpts=(1,1,1))
### kpp.write_file("KPP")

def converg_encut(encut=500,mat=None):
    en1=-10000
    encut1=encut
    convg_encut1=False
    convg_encut2=False
    
    while  convg_encut2 !=True:
    #while convg_encut1 !=True and  convg_encut2 !=True:
        tol=0.001          #change 0.001
        encut_list=[]
        encut_list.append(encut)
        length=10
        encut1=encut+50
        incar_dict = dict(
            PREC = 'Accurate',
            ENCUT = encut,
            ISMEAR = 0,
            IBRION=2,

            GGA = 'BO',
            PARAM1 = 0.1833333333,
            PARAM2 = 0.2200000000,
            LUSE_VDW = '.TRUE.',
            AGGAC = 0.0000,

            EDIFF = '1E-7',
            NSW = 1,
            NELM = 400,
            NPAR = 4,
            LCHARG = '.FALSE.',
            LWAVE = '.FALSE.' )
        incar = Incar.from_dict(incar_dict)
        kpoints=Auto_Kpoints(mat=mat,length=length)
        print ("running smart_converge for",str(mat.comment)+str('-')+str('ENCUT')+str('-')+str(encut))
        en2,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('ENCUT')+str(mat.comment)+str('-')+str(encut))
        while abs(en2-en1)>tol:
           en1=en2
           encut1=encut+50
           encut_list.append(encut)
           print("Incrementing encut",encut)
           incar_dict = dict(
               PREC = 'Accurate',
               IBRION=2,
               ENCUT = encut1,
               ISMEAR = 0,
               EDIFF = '1E-7',
               NSW = 1,
               GGA = 'BO',
               PARAM1 = 0.1833333333,
               PARAM2 = 0.2200000000,
               LUSE_VDW = '.TRUE.',
               AGGAC = 0.0000,
               NELM = 400,


               NPAR = 4,
               LCHARG = '.FALSE.',
               LWAVE = '.FALSE.' )
           incar = Incar.from_dict(incar_dict)
           print ("running smart_converge for",str(mat.comment)+str('-')+str('ENCUT')+str('-')+str(encut))
           en2,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('ENCUT')+str(mat.comment)+str('-')+str(encut))
        convg_encut1=True


# Some extra points to check
        print ("Some extra points to check for ENCUT")
        
        encut2=encut1+50
        incar['ENCUT']=encut2
        en3,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('ENCUT')+str(mat.comment)+str('-')+str(encut2))
       
        encut3=encut2+50
        incar['ENCUT']=encut3
        en4,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('ENCUT')+str(mat.comment)+str('-')+str(encut3))



        encut4=encut3+50
        incar['ENCUT']=encut4
        en5,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('ENCUT')+str(mat.comment)+str('-')+str(encut4))

        encut5=encut4+50
        incar['ENCUT']=encut5
        en6,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('ENCUT')+str(mat.comment)+str('-')+str(encut5))

        encut6=encut5+50
        incar['ENCUT']=encut6
        en7,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('ENCUT')+str(mat.comment)+str('-')+str(encut6))

        #if en3-en2>tol or en4-en2>tol or en5-en2>tol or en6-en2>tol or en7-en2>tol:
        #if abs(en3-en2)>tol and abs(en4-en2)>tol and abs(en5-en2)>tol and abs(en6-en2)>tol and abs(en7-en2)>tol:
        if abs(en3-en2)>tol or abs(en4-en2)>tol or abs(en5-en2)>tol or abs(en6-en2)>tol or abs(en7-en2)>tol:

           en1=en3
           encut=encut1
           fen=open("EXTRA_ENCUT","w")
           line=str("Extra ENCUT needed ")+str(encut)+'\n'
           fen.write(line)
           fen.close()
        else:
          print ("ENCUT convergence achieved for ",mat.comment,encut)
          convg_encut2=True
    return encut

    
def converg_kpoints(length=0,mat=None):
    en1=-10000
    encut=550
    convg_kp1=False
    convg_kp2=False
    length1=length
    kp_list=[]
    while   convg_kp2 !=True:
    #while convg_kp1 !=True and  convg_kp2 !=True:
        tol=0.001          #change 0.001
        incar_dict = dict(
            PREC = 'Accurate',
            ENCUT = encut,
            ISMEAR = 0,
            IBRION=2,
            EDIFF = '1E-7',
            NSW = 1,
            NELM = 400,
            GGA = 'BO',
            PARAM1 = 0.1833333333,
            PARAM2 = 0.2200000000,
            LUSE_VDW = '.TRUE.',
            AGGAC = 0.0000,

            NPAR = 4,
            LCHARG = '.FALSE.',
            LWAVE = '.FALSE.' )
        incar = Incar.from_dict(incar_dict)
        length1=length1+5
        print ("Incrementing length",length1)
        kpoints=Auto_Kpoints(mat=mat,length=length1)
        mesh=kpoints.kpts[0]
        if mesh not in kp_list:
           kp_list.append(mesh)
           en2,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('KPOINTS')+str(mat.comment)+str('-')+str(length1))



           while abs(en2-en1)>tol:
              en1=en2
              print ("Incrementing length",length1)
              incar_dict = dict(
                  PREC = 'Accurate',
                  IBRION=2,
                  ENCUT = encut,
                  ISMEAR = 0,
                  EDIFF = '1E-7',
                  NSW = 1,
                  GGA = 'BO',
                  PARAM1 = 0.1833333333,
                  PARAM2 = 0.2200000000,
                  LUSE_VDW = '.TRUE.',
                  AGGAC = 0.0000,

                  NELM = 400,
                  NPAR = 4,
                  LCHARG = '.FALSE.',
                  LWAVE = '.FALSE.' )
              incar = Incar.from_dict(incar_dict)
              while mesh in kp_list:
                 length1=length1+5
                 ##Assuming you are not super unlucky
                 kpoints=Auto_Kpoints(mat=mat,length=length1)
                 mesh=kpoints.kpts[0]
                    
              kpoints=Auto_Kpoints(mat=mat,length=length1)
              mesh=kpoints.kpts[0]
              if mesh not in kp_list:
                 kp_list.append(mesh)
                 en2,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('KPOINTS')+str(mat.comment)+str('-')+str(length1))
              else:
                 length1=length1+5
                 ##Assuming you are not super unlucky
                 kpoints=Auto_Kpoints(mat=mat,length=length1)
                 mesh=kpoints.kpts[0]
                 kp_list.append(mesh)
                 en2,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('KPOINTS')+str(mat.comment)+str('-')+str(length1))
           convg_kp1=True


# Some extra points to check
           print ("Some extra points to check for KPOINTS")
           length3=length1+5
           kpoints=Auto_Kpoints(mat=mat,length=length3)
           mesh=kpoints.kpts[0]
           kp_list.append(mesh)
           en3,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('KPOINTS')+str(mat.comment)+str('-')+str(length3))
       
           length4=length3+5
           kpoints=Auto_Kpoints(mat=mat,length=length4)
           mesh=kpoints.kpts[0]
           kp_list.append(mesh)
           en4,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('KPOINTS')+str(mat.comment)+str('-')+str(length4))


           length5=length4+5
           kpoints=Auto_Kpoints(mat=mat,length=length5)
           mesh=kpoints.kpts[0]
           kp_list.append(mesh)
           en5,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('KPOINTS')+str(mat.comment)+str('-')+str(length5))



           length6=length5+5
           kpoints=Auto_Kpoints(mat=mat,length=length6)
           mesh=kpoints.kpts[0]
           kp_list.append(mesh)
           en6,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('KPOINTS')+str(mat.comment)+str('-')+str(length6))
           length7=length6+5
           kpoints=Auto_Kpoints(mat=mat,length=length7)
           mesh=kpoints.kpts[0]
           kp_list.append(mesh)
           en7,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('KPOINTS')+str(mat.comment)+str('-')+str(length7))



           #if en3-en2>tol or en4-en2>tol or en5-en2>tol or en6-en2>tol or en7-en2>tol:
           #if abs(en3-en2)>tol and abs(en4-en2)>tol and abs(en5-en2)>tol and abs(en6-en2)>tol and abs(en7-en2)>tol:
           if abs(en3-en2)>tol or abs(en4-en2)>tol or abs(en5-en2)>tol or abs(en6-en2)>tol or abs(en7-en2)>tol:
              fkp=open("EXTRA_KPOINTS","w")
              line=str("Extra KPOINTS needed ")+str(length1)+'\n'
              fkp.write(line)
              line=str("en2 length1 ")+str (" ")+str(en2)+str(" ")+str(length1)+'\n'
              fkp.write(line)
              line=str("en3 length3 ")+str (" ")+str(en3)+str(" ")+str(length3)+'\n'
              fkp.write(line)
              line=str("en4 length4 ")+str (" ")+str(en4)+str(" ")+str(length4)+'\n'
              fkp.write(line)
              line=str("en5 length5 ")+str (" ")+str(en5)+str(" ")+str(length5)+'\n'
              fkp.write(line)
              fkp.close()
              en1=en3
              length1=length3
           else:
              print ("KPOINTS convergence achieved for ",mat.comment,length1)
              convg_kp2=True


    return length1

def smart_converge(mat=None,band_str=True,elast_prop=True,optical_prop=True,mbj_prop=True,spin_orb=False,Raman_calc=False,surf_en=False,def_en=False):
    encut=converg_encut(encut=500,mat=mat)
    leng= converg_kpoints(length=0,mat=mat)
    kpoints=Auto_Kpoints(mat=mat,length=leng)
    isif=2
    commen=str(mat.comment)
    lcharg='.FALSE.' 
    if commen.split('@')[0] =='bulk' :
       isif=3
       lcharg='.TRUE.' 
    if commen.split('@')[0] == 'sbulk':
       isif=3
    incar_dict = dict(
        PREC = 'Accurate',
        ENCUT = encut,
        ISMEAR = 0,
        EDIFF = '1E-7',
        EDIFFG = '-1E-3',
        ISIF = 3,
        GGA = 'BO',
        PARAM1 = 0.1833333333,
        PARAM2 = 0.2200000000,
        LUSE_VDW = '.TRUE.',
        AGGAC = 0.0000,


        NEDOS = 5000,
        IBRION = 2,
        NSW = 500,   #change 400
        NELM = 500,  #change 400
        LORBIT = 11,
        LVTOT = '.TRUE.',
        LVHAR = '.TRUE.',
        ISPIN = 2,
        NPAR = 4,
        LCHARG = lcharg,
        LWAVE = '.FALSE.' )
    incar = Incar.from_dict(incar_dict)
    try:
       if mat.comment.startswith('Mol'):
          incar.update({"ISIF": '2'})
    except:  
         print ("Mol error")
         pass
    #if commen.startswith('Surf-') :
    #   pol=check_polar(mat_f.structure)
    #   if pol==True:
    #        ase_atoms = AseAtomsAdaptor().get_atoms(mat_f.structure)
    #        COM=ase_atoms.get_center_of_mass(scaled=True)
    #        incar.update({"LDIPOL": '.TRUE.',"IDIPOL":4,"ISYM": 0,"DIPOL":COM})
    print ("running smart_converge for",str(mat.comment)+str('-')+str('MAIN-RELAX'))
    cwd=str(os.getcwd()) 
    en2,contc=run_job(mat=mat,incar=incar,kpoints=kpoints,jobname=str('MAIN-RELAX')+str('-')+str(mat.comment)) 
    os.chdir(cwd)
    path=str(contc.split('/CONTCAR')[0])+str('/vasprun.xml')
    v=open(path,"r").readlines()
    for line in v:
           if "NBANDS" in  line:
               nbands=int(line.split(">")[1].split("<")[0])
               print ("nbands=",nbands)
               break
    strt=Structure.from_file(contc)
    mat_f=Poscar(strt)
    mat_f.comment=str(mat.comment)
    if band_str==True:
       incar_dict = dict(
           PREC = 'Accurate',
           ENCUT = encut,
           NBANDS=int(nbands)+10,
           ISMEAR = 0,
           EDIFF = '1E-7',
           GGA = 'BO',
           PARAM1 = 0.1833333333,
           PARAM2 = 0.2200000000,
           LUSE_VDW = '.TRUE.',
           AGGAC = 0.0000,


           LCHARG = '.FALSE.',
           NEDOS = 5000,
           ISPIN = 2,
           ISIF = 2,
           IBRION = 1,
           NELM = 400,
           LORBIT = 11,
           NPAR = 4,
           LWAVE = '.FALSE.' )
       incar = Incar.from_dict(incar_dict)
       user_incar_settings={"EDIFF":1E-6,"ISIF":2,"NSW":0,"LORBIT":11,"ENCUT":encut,"LWAVE":'.FALSE.',"PREC":'Accurate'}
       #mpvis = MPNonSCFVaspInputSet(user_incar_settings=incar)
       kpath = HighSymmKpath(mat_f.structure)
       frac_k_points, k_points_labels = kpath.get_kpoints(line_density=20,coords_are_cartesian=False)
       kpoints = Kpoints(comment="Non SCF run along symmetry lines",style=Kpoints.supported_modes.Reciprocal,num_kpts=len(frac_k_points),kpts=frac_k_points, labels=k_points_labels,kpts_weights=[1] * len(frac_k_points))
 

       try: 
           print ("running MAIN-BAND")
           kpoints=mpvis.get_kpoints(mat_f.structure)
           en2B,contcB=run_job(mat=mat_f,incar=incar,kpoints=kpoints,jobname=str('MAIN-BAND')+str('-')+str(mat_f.comment))  
          # kpoints=mpvis.get_kpoints(mat_f.structure)
          # en2B,contcB=run_job(mat=mat_f,incar=incar,kpoints=kpoints,jobname=str('MAIN-BAND')+str('-')+str(mat_f.comment))  
       except:
           print ("No band str calc.")
           if str(os.getcwd)!=cwd:
                print ("Changing directory")
                line=str("cd ")+str(cwd)
                os.chdir(cwd)
                print (os.getcwd())

           pass
    os.chdir(cwd)
    if surf_en==True:
       incar_dict = dict(
           PREC = 'Accurate',
           ENCUT = encut,
           ISMEAR = 0,
           EDIFF = '1E-7',
           GGA = 'BO',
           PARAM1 = 0.1833333333,
           PARAM2 = 0.2200000000,
           LUSE_VDW = '.TRUE.',
           AGGAC = 0.0000,


           LCHARG = '.FALSE.',
           NEDOS = 5000,
           ISIF = 2,
           IBRION = 1,
           NELM = 400,
           NSW = 200,
           LORBIT = 11,
           NPAR = 4,
           LWAVE = '.FALSE.' )
       incar = Incar.from_dict(incar_dict)
       surf=surfer(mat=mat_f.structure,layers=3)
       for i in surf:


         try: 
           print ("running MAIN-BAND")
           kpoints=Auto_Kpoints(mat=i,length=leng)
           en2s,contcs=run_job(mat=i,incar=incar,kpoints=kpoints,jobname=str('Surf_en-')+str(i.comment)+str('-')+str(mat_f.comment))  
          # kpoints=mpvis.get_kpoints(mat_f.structure)
          # en2B,contcB=run_job(mat=mat_f,incar=incar,kpoints=kpoints,jobname=str('MAIN-BAND')+str('-')+str(mat_f.comment))  
         except:

           pass
    os.chdir(cwd)
    if def_en==True:
       incar_dict = dict(
           PREC = 'Accurate',
           ENCUT = encut,
           ISMEAR = 0,
           EDIFF = '1E-7',
           GGA = 'BO',
           PARAM1 = 0.1833333333,
           PARAM2 = 0.2200000000,
           LUSE_VDW = '.TRUE.',
           AGGAC = 0.0000,


           LCHARG = '.FALSE.',
           NEDOS = 5000,
           ISIF = 2,
           IBRION = 1,
           NELM = 400,
           NSW = 200,
           LORBIT = 11,
           NPAR = 4,
           LWAVE = '.FALSE.' )
       incar = Incar.from_dict(incar_dict)
       #surf=surfer(mat=mat_f.structure,layers=3)
       vac=vac_antisite_def_struct_gen(cellmax=3,struct=mat_f.structure)
       for i in vac:


         try: 
           print ("running MAIN-vac")
           kpoints=Auto_Kpoints(mat=i,length=leng)
           en2d,contcd=run_job(mat=i,incar=incar,kpoints=kpoints,jobname=str('Def_en-')+str(i.comment)+str('-')+str(mat_f.comment))  
          # kpoints=mpvis.get_kpoints(mat_f.structure)
          # en2B,contcB=run_job(mat=mat_f,incar=incar,kpoints=kpoints,jobname=str('MAIN-BAND')+str('-')+str(mat_f.comment))  
         except:

           pass
    os.chdir(cwd)
    #surf=surfer(mat=strt,layers=layers)
    #surf=surfer(mat=strt,layers=layers)
    if spin_orb==True:
       #chg_file=str(contc).replace('CONTCAR','CHGCAR')
       #print ('chrfile',chg_file)
       #shutil.copy2(chg_file,'./')
       incar_dict = dict(
           PREC = 'Accurate',
           ENCUT = encut,

           GGA_COMPAT = '.FALSE.',
           LWANNIER90 = '.TRUE.',
           LSORBIT = '.TRUE.',
           ISIF = 2,
           IBRION = 1,
           ISMEAR = 0,
           ISYM = 0,
           EDIFF = '1E-7',

           NEDOS = 5000,
           LCHARG = '.FALSE.',
           NELM = 400,
           LORBIT = 11,
           NPAR = 4,
           LWAVE = '.FALSE.' )
       incar = Incar.from_dict(incar_dict)
       sg_mat = SpacegroupAnalyzer(mat_f.structure)
       mat_cvn = sg_mat.get_conventional_standard_structure()
       mat_cvn.sort()
       kpoints=Auto_Kpoints(mat=Poscar(mat_cvn),length=leng/2)
       try:
          en2S,contcS=run_job(mat=Poscar(mat_cvn),incar=incar,kpoints=kpoints,jobname=str('MAIN-SOC')+str('-')+str(mat_f.comment)) 
       except:
           pass 
    os.chdir(cwd)
    if optical_prop==True:

       incar_dict = dict(
           PREC = 'Accurate',
           ENCUT = encut,
           NBANDS = 3*int(nbands),
           GGA = 'BO',
           PARAM1 = 0.1833333333,
           PARAM2 = 0.2200000000,
           LUSE_VDW = '.TRUE.',
           AGGAC = 0.0000,


           LOPTICS = '.TRUE.',
           ISMEAR = 0,
           EDIFF = '1E-7',

           NEDOS = 5000,
           LCHARG = '.FALSE.',
           ISIF = 2,
           IBRION = 1,
           NELM = 400,
           LORBIT = 11,
           NPAR = 4,
           LWAVE = '.FALSE.' )
       incar = Incar.from_dict(incar_dict)
       kpoints=Auto_Kpoints(mat=mat_f,length=leng)
       try:
           en2OP,contcOP=run_job(mat=mat_f,incar=incar,kpoints=kpoints,jobname=str('MAIN-OPTICS')+str('-')+str(mat_f.comment)) 
       except:
           pass 
    os.chdir(cwd)
    if mbj_prop==True:

       incar_dict = dict(
           PREC = 'Accurate',
           ENCUT = encut,
           NBANDS = 3*int(nbands),
           METAGGA = 'MBJ',


           LOPTICS = '.TRUE.',
           ISMEAR = 0,
           EDIFF = '1E-6',

           NEDOS = 5000,
           LCHARG = '.FALSE.',
           ISYM = 0,
           SIGMA = 0.1,
           NELM = 500,
           LORBIT = 11,
           NPAR = 4,
           LWAVE = '.FALSE.' )
       incar = Incar.from_dict(incar_dict)
       kpoints=Auto_Kpoints(mat=mat_f,length=leng)
       try:
           en2OP,contcOP=run_job(mat=mat_f,incar=incar,kpoints=kpoints,jobname=str('MAIN-MBJ')+str('-')+str(mat_f.comment))
       except:
           pass
    os.chdir(cwd)

    if elast_prop==True:
       incar_dict = dict(
           PREC = 'Accurate',
           ENCUT = float(encut)*1.3,
           ISMEAR = 0,
           GGA = 'BO',
           PARAM1 = 0.1833333333,
           PARAM2 = 0.2200000000,
           LUSE_VDW = '.TRUE.',
           AGGAC = 0.0000,


           ISIF = 3,
           POTIM = 0.015,
           NEDOS = 5000,
           EDIFF = '1E-7',
           IBRION = 6,
           NELM = 400,
           NPAR = 24,
           LWAVE = '.FALSE.' )
       incar = Incar.from_dict(incar_dict)
       sg_mat = SpacegroupAnalyzer(mat_f.structure)
       mat_cvn = sg_mat.get_conventional_standard_structure()
       mat_cvn.sort()
       kpoints=Auto_Kpoints(mat=Poscar(mat_cvn),length=leng)
       try:
          en2E,contcE=run_job(mat=Poscar(mat_cvn),incar=incar,kpoints=kpoints,jobname=str('MAIN-ELASTIC')+str('-')+str(mat_f.comment)) 
       except:
           pass 
    os.chdir(cwd)
    if Raman_calc==True:
        Raman(strt=mat_f,encut=encut,length=leng)
    os.chdir(cwd)
    return en2,mat_f
#####################################################




def bandstr(contc=None,kpoints=None,encut=500):
    #if commen.split('@')[0] =='bulk' :
       incar_dict = dict(
           PREC = 'Accurate',
           ENCUT = encut,

           ISMEAR = 0,
           EDIFF = '1E-4',
           LCHARG = lcharg,
           NEDOS = 5000,
           ISPIN = 2,
           ISIF = 2,
           IBRION = 1,
           NELM = 400,
           GGA = 'BO',
           PARAM1 = 0.1833333333,
           PARAM2 = 0.2200000000,
           LUSE_VDW = '.TRUE.',
           AGGAC = 0.0000,


           LORBIT = 11,
           NPAR = 4,
           LWAVE = '.FALSE.' )
       incar = Incar.from_dict(incar_dict)
       user_incar_settings={"EDIFF":1E-6,"ISIF":2,"NSW":0,"LORBIT":11,"ENCUT":encut,"LWAVE":'.FALSE.',"PREC":'Accurate'}
       mpvis = MPNonSCFVaspInputSet(user_incar_settings=incar)
       final_str=Structure.from_file(contc)
       if kpoints!=None:
          print ("Band structure calculation for user input")
       else:

           kpoints=mpvis.get_kpoints(final_str)
       print ("running smart_converge for",str(mat_f.comment)+str('-')+str('BAND')+str('-')+str(den))
       try:
          en2B,contcB=run_job(mat=mat_f,incar=incar,kpoints=kpoints,jobname=str('MAIN-BAND')+str('-')+str(mat_f.comment))  
       except:
           pass

def elastic_prop(mat_f=None,kpoints=None,encut=500):


       path=str(mat_f.split('/CONTCAR')[0])+str('/vasprun.xml')
       v=open(path,"r").readlines()
       for line in v:
           if "NBANDS" in  line:
               nbands=int(line.split(">")[1].split("<")[0])
               print ("nbands=",nbands)
               break
       incar_dict = dict(
           PREC = 'Accurate',
           ENCUT = encut,
           ISMEAR = 0,
           ISIF = 3,
           GGA = 'BO',
           PARAM1 = 0.1833333333,
           PARAM2 = 0.2200000000,
           LUSE_VDW = '.TRUE.',
           AGGAC = 0.0000,


           POTIM = 0.015,
           NEDOS = 5000,
           EDIFF = '1E-6',
           IBRION = 6,
           NELM = 400,
           NPAR = 16,
           LWAVE = '.FALSE.' )
       incar = Incar.from_dict(incar_dict)
       #v.close()
       final_str=Structure.from_file(contc)
       
       #num_kpoints = (den)*mat.structure.lattice.reciprocal_lattice.volume
       #kpoints = Kpoints.automatic_density(mat.structure, num_kpoints * mat.structure.num_sites,force_gamma=True)
       #kpoints = Kpoints.gamma_automatic(kpts=kpts)
       print ("running smart_converge for",str(mat_f.comment)+str('-')+str('ELASTIC'))
       sg_mat = SpacegroupAnalyzer(mat_f.structure)
       mat_cvn = sg_mat.get_conventional_standard_structure()
       mat_cvn.sort()
       en2E,contcE=run_job(mat=Poscar(mat_cvn),incar=incar,kpoints=kpoints,jobname=str('MAIN-ELASTIC')+str('-')+str(mat_f.comment))  
       #try:
       #   phonts(mat_f)
       #except:
       #   pass





def optical_prop(mat_f=None,kpoints=None,encut=500,nbands=100):
       incar_dict = dict(
           PREC = 'Accurate',
           ENCUT = encut,
           NBANDS = 3*int(nbands),
           LOPTICS = '.TRUE.',
           ISMEAR = 0,
           EDIFF = '1E-4',

           NEDOS = 5000,
           LCHARG = '.FALSE.',
           ISIF = 2,
           GGA = 'BO',
           PARAM1 = 0.1833333333,
           PARAM2 = 0.2200000000,
           LUSE_VDW = '.TRUE.',
           AGGAC = 0.0000,


           IBRION = 1,
           NELM = 400,
           LORBIT = 11,
           NPAR = 4,
           LWAVE = '.FALSE.' )
       incar = Incar.from_dict(incar_dict)
       #num_kpoints = (den)*mat.structure.lattice.reciprocal_lattice.volume
       #kpoints = Kpoints.automatic_density(mat.structure, num_kpoints * mat.structure.num_sites,force_gamma=True)
       kpoints = Kpoints.gamma_automatic(kpts=kpts)
       en2OP,contcOP=run_job(mat=mat_f,incar=incar,kpoints=kpoints,jobname=str('MAIN-OPTICS')+str('-')+str(mat_f.comment))  

def smart_vac(strt=None):
    vac_arr=[]
    sg_mat = SpacegroupAnalyzer(strt)
    mat_cvn = sg_mat.get_conventional_standard_structure()
    mat_cvn.sort()

    cellmax=1 #int(mat_cvn.composition.num_atoms)+int(mat_cvn.ntypesp)#5
    ase_atoms = AseAtomsAdaptor().get_atoms(mat_cvn)
    ase_atoms=ase_atoms*(cellmax,cellmax,cellmax)
    #if len(ase_atoms) >200:
    #   cellmax=1
    #else:
    #   cellmax=2
    #cellmax=int(mat_cvn.composition.num_atoms)+int(mat_cvn.ntypesp)#5
    print ("type of trt is= celmmax",type(strt),cellmax)
    try:
       print ("int(strt.composition.num_atoms)",int(strt.composition.num_atoms))
       print (int(strt.ntypesp))
    except:
       pass
    #cellmax=int(strt.composition.num_atoms)+int(strt.ntypesp)
    vac_done=0
    tol=0.1   #change 0.1
    vac=vac_antisite_def_struct_gen(cellmax=cellmax,struct=strt)
    def_list=[100000  for y in range(len(vac)-1)]
    while vac_done !=1:
        vac=vac_antisite_def_struct_gen(cellmax=cellmax,struct=strt)
        if vac not in vac_arr:
           vac_arr.append(vac)
           print ("in smart_vac(strt=None), cellmax,vac,vac_done=",cellmax,vac,vac_done)
           def_list2,header_list=def_energy(vac=vac)
           diff=matrix(def_list)-matrix(def_list2)
           diff_arr=np.array(diff).flatten()
           print ("in smart_vac(strt=None diff_arr=",diff_arr)
           if any(diff_arr)>tol:
          # for el in diff_arr:
          #     if abs(el)>tol :
                  # print ("in smart_vac(strt=None abs_el=",abs(el))
                   vac_done=0
                   cellmax=cellmax+1
                   ase_atoms = AseAtomsAdaptor().get_atoms(mat_cvn)
                   ase_atoms=ase_atoms*(cellmax,cellmax,cellmax)
                   if len(ase_atoms) >100:
                      vac_done=1
                   def_list=def_list2
           else:
               vac_done=1
#        cellmax=cellmax+1
    return def_list,header_list

def def_energy(vac=[]):
    def_list=[]
    header_list=[]
    fi=str('DEF.INFO')
    f=open(fi,'w')
    for v in vac:
        enp,contc=smart_converge(mat=v)
        #enp,contc=run_job(mat=v,incar=incar,kpoints=kpoints,jobname=str(v.comment))
        strt=Structure.from_file(contc)
        comm=str(v.comment)
        print ("running def_energy for =",comm)
        header_list.append(comm)
        if comm.split('@')[0]=='bulk':
           gs_energy=float(enp)*float(strt.composition.num_atoms)
           print ("in def_energy gs_energy for",comm, gs_energy)
        else:
           chem_pot=sum_chem_pot(strt)
           if comm.startswith("intl"):
              chem_pot=0.0-float(chem_pot)
           def_en=(float(enp)*float(strt.composition.num_atoms))-float(gs_energy)+chem_pot
           print ("in def_energy def_en for",comm, def_en,"chem_pot",chem_pot)
           def_list.append(def_en)
           print (v.comment,'=',def_en)
           line= str(v.comment)+str('=')+str(def_en)+'\n'
           f.write(line)
    f.close()
    return def_list,header_list

def smart_surf(strt=None):
    sg_mat = SpacegroupAnalyzer(strt)
    mat_cvn = sg_mat.get_conventional_standard_structure()
    mat_cvn.sort()
    layers=2
    indices = get_symmetrically_distinct_miller_indices(mat_cvn, 1)
    ase_atoms = AseAtomsAdaptor().get_atoms(mat_cvn)
    for i in indices:
        ase_slab = surface(ase_atoms, i, layers)
        ase_slab.center(vacuum=15, axis=2)
        if len(ase_slab) < 50:
           layers=3
    surf_arr=[]
    surf_done=0
    tol=0.1 #change 0.01
    surf=surfer(mat=strt,layers=layers)
    surf_list=[100000  for y in range(len(surf)-1)]
    print ("in smart_surf :surf,surf_list=",surf, surf_list)
    while surf_done !=1:
        layers=layers+1
        indices = get_symmetrically_distinct_miller_indices(mat_cvn, 1)
        ase_atoms = AseAtomsAdaptor().get_atoms(mat_cvn)
        for i in indices:
            ase_slab = surface(ase_atoms, i, layers)
            ase_slab.center(vacuum=15, axis=2)
            if len(ase_slab) > 100:
               surf_done=1
            if (ase_slab.get_cell()[2][2]) > 40:
               surf_done=1
        surf=surfer(mat=strt,layers=layers)
        if surf not in surf_arr:
           surf_arr.append(surf)
           surf_list2,surf_header_list=surf_energy(surf=surf)
           print ("in smart_surf :surf2,surf_list2=",surf_list2,surf_header_list)
           diff=matrix(surf_list)-matrix(surf_list2)
           print ("in smart_surf :surf3,surf_list3=",matrix(surf_list),matrix(surf_list2))
           diff_arr=np.array(diff).flatten()
           if any(diff_arr)>tol:
           #for el in diff_arr:
           #    if abs(el)>tol :
           #        print ("in smart_surf :abs el=",abs(el))
                   surf_done=0
                   surf_list=surf_list2
           else:
                surf_done=1
    return surf_list,surf_header_list
def surf_energy(surf=[]):
    surf_list=[]
    surf_header_list=[]
    fi=str('SURF.INFO')
    f=open(fi,'w')
    for s in surf:
        enp,contc=smart_converge(mat=s)
        strt=Structure.from_file(contc)
        m=strt.lattice.matrix
        if s.comment.split('@')[0]=='sbulk':

           gs_energy=enp
           print ("in surf_energy gs_energy for",s.comment, gs_energy)
        else:
           surf_en=16*(float(enp)*(strt.composition.num_atoms)-float(strt.composition.num_atoms)*float(gs_energy))/(2*np.linalg.norm(np.cross(m[0], m[1])))
           print ("in surf_energy surf_en for",s.comment, surf_en)
           surf_list.append(surf_en)
           print (s.comment,'=',surf_en)
           line= str(s.comment)+str('=')+str(surf_en)+'\n'
           f.write(line)
    f.close()
    return surf_list,surf_header_list

def make_big(poscar=None):
    struct=poscar.structure
    comm=poscar.comment
    a, b, c = struct.lattice.abc
    size=11
    struct.make_supercell([int(float(size)/float(a))+1,int(float(size)/float(b))+1,int(float(size)/float(c))+1])
    big=Poscar(struct)
    big.comment=str(comm)
    return big


def get_smart_surf_def(mat=None):
    #with MPRester() as mp:
    #     strt = mp.get_structure_by_material_id(mpid)
    mat=Poscar.from_file("POSCAR")
#
    #     mpid=mpid.replace('-','_')
    #     mpid=str('bulk@')+str(mpid)
    #     mat.comment=mpid

    print ("Running Main Relax")
    en,contc=smart_converge(mat=mat)
    relaxed_str=Poscar.from_file(contc)
    commen=str(relaxed_str.comment)
    kpfile=str("MAIN-RELAX-")+str(commen)+str('/KPOINTS')
    kp=Kpoints.from_file(kpfile)
    incfile=str("MAIN-RELAX-")+str(commen)+str('/INCAR')
    inc=Incar.from_file(incfile)
    encut=inc['ENCUT']

    #pos1=Poscar.from_file("POSCAR")
    #kp1=Kpoints.from_file("KPOINTS")
    #print kp1.kpts
    tag=True
    kppa=1
    while tag:
      kpoints = Kpoints.automatic_density(relaxed_str.structure, kppa,force_gamma=True)
      #print kpoints.kpts[0],kp1.kpts[0]
      kppa=kppa+1
      if (kpoints.kpts[0])>=(kp1.kpts[0]):
         print (kpoints.kpts,kp1.kpts)
         tag=False
      print ("kpp=",kpp)

    kpoints = Kpoints.automatic_density(relaxed_str.structure, kppa,force_gamma=True)
    elastic_prop(mat_f=relaxed_str,kpoints=kpoints,encut=encut)


    bandstr(contc=relaxed_str,encut=encut)
    optical_prop(mat_f=relaxed_str,kpoints=kpoints,encut=encut,nbands=40)
    big=make_big(relaxed_str)
    try:
        Raman(strt=big,encut=encut,kpoints=kp)
    except:
        print ("Failed Raman")
        pass
#get_smart_surf_def(mat='POSCAR')
def main_func(mpid='',mat=None,enforc_cvn=False):
    if mpid !='':
       data = loadfn(json_dat, cls=MontyDecoder)
       for d in data:
            mmpid= str(d['mp_id'])
            if mmpid==mpid: 
               ini= (d['ini_structure'])
               fin= (d['fin_structure'])

               break

       strt = fin#mp.get_structure_by_material_id(mpid)
       sg_mat = SpacegroupAnalyzer(strt)
       mat_cvn = sg_mat.get_conventional_standard_structure()
       mat_cvn.sort()
       if int(strt.composition._natoms)==int(mat_cvn.composition._natoms) and enforc_cvn==True:
               mat= Poscar(mat_cvn)
       else:
                mat=Poscar(strt)



       mpid=mpid.replace('-','_')
       mpid=str('bulk@')+str(mpid)
       mat.comment=mpid

    en,final=smart_converge(mat=mat)
    print (en,final)
#struct=Structure.from_file("POSCAR")
#pos=Poscar.from_file('POSCAR')
#pos.comment='Surf-mp-C'
#main_func(mat=pos)

#main_func(mpid='mp-149')
#main_func(mpid='mp-605')



from monty.json import MontyEncoder, MontyDecoder
from monty.serialization import loadfn, dumpfn

"""
 Download 3D material data from https://figshare.com/articles/jdft_3d-7-7-2018_json/6815699
 Download 2D material data from https://figshare.com/articles/jdft_2d-7-7-2018_json/6815705 

 Website: https://jarvis.nist.gov
 https://www.nature.com/articles/s41598-017-05402-0
 https://www.nature.com/articles/sdata201882
 https://journals.aps.org/prb/abstract/10.1103/PhysRevB.98.014107
"""


if __name__=='__main__':
 d=loadfn('jdft_3d.json',cls=MontyDecoder)
 #d=loadfn('jdft_2d.json',cls=MontyDecoder)

 print ("Total size",len(d))
 for i in d:
    print ("JARVIS id", i['jid'],"link",str("https://www.ctcms.nist.gov/~knc6/jsmol/")+str(i['jid'])+str(".html"))
    print ("Materials project id"), i['mpid'],"link",str("https://www.materialsproject.org/materials/")+str(i['mpid'])
    print ("final energy eV",i['fin_en'])
    print ("magnetic moment, Bohr Mag.",i['magmom'])
    print ("optb88vdw bandgap eV", i["op_gap"])
    print ("Tran-Blaha MBJ bandgap  eV",i["mbj_gap"])
    print ("static dielectric function in x,optb88vdw", i["epsx"]) #Careful about 2D
    print ("static dielectric function in x,MBJ", i["mepsx"]) #Careful about 2D
    print ("bulk modulus GPa", i["kv"]) #Careful about 2D
    print ("Shear modulus GPa", i["gv"]) #Careful about 2D
    break



from __future__ import  unicode_literals, print_function

"""
Helper function for running LAMMPS
Used for defects, surface and phonon calculations
"""

from monty.json import MontyEncoder
from numpy import matrix
import time
from ase import *
import numpy as np
from phonopy import Phonopy
from phonopy.file_IO import parse_FORCE_CONSTANTS, write_FORCE_CONSTANTS
from phonopy.structure.atoms import Atoms as PhonopyAtoms

import glob
from pymatgen.core.structure import Structure
from pymatgen.io.vasp.inputs import Incar, Poscar
from pymatgen.core.surface import  Slab, SlabGenerator, generate_all_slabs,get_symmetrically_distinct_miller_indices
from ase.lattice.surface import surface
from pymatgen.ext.matproj import MPRester
import operator
from pymatgen.core.lattice import Lattice
from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
from jarvis.lammps.Surf_Def import vac_antisite_def_struct_gen,pmg_surfer,surfer
import numpy as np,time,json
import sys,os,subprocess,socket
from pymatgen.io.ase import AseAtomsAdaptor
from ase.calculators.lammpsrun import LAMMPS, Prism
import sys,zipfile
import fortranformat as fform
from pymatgen.core.structure import Structure
from phonopy.structure.atoms import Atoms as PhonopyAtoms

def get_phonopy_atoms(mat=None):
    """
    Helper function to convert pymatgen structure object to phonopy atoms
    """
    symbols = [str(site.specie.symbol) for site in mat]
    positions = [site.coords for site in mat]
    cell = mat.lattice.matrix
    p=PhonopyAtoms(symbols=symbols, positions=positions, pbc=True, cell=cell)
    return p
def ZipDir(inputDir, outputZip,contents=[]):
    """
    Zip up a directory and preserve symlinks and empty directories
    """
    zipOut = zipfile.ZipFile(outputZip, 'w', compression=zipfile.ZIP_DEFLATED)
    tmp=contents
    rootLen = len(os.path.dirname(inputDir))
    def _ArchiveDirectory(parentDirectory):
        #contents = os.listdir(parentDirectory)
        contents = tmp#['init.mod', 'potential.mod', 'in.elastic', 'data',  'log.lammps', 'restart.equil','data0' ]
        #store empty directories
        if not contents:
            #http://www.velocityreviews.com/forums/t318840-add-empty-directory-using-zipfile.html
            archiveRoot = parentDirectory[rootLen:].replace('\\', '/').lstrip('/')
            zipInfo = zipfile.ZipInfo(archiveRoot+'/')
            zipOut.writestr(zipInfo, '')
        for item in contents:
            fullPath = os.path.join(parentDirectory, item)
            if os.path.isdir(fullPath) and not os.path.islink(fullPath):
                _ArchiveDirectory(fullPath)
            else:
                archiveRoot = fullPath[rootLen:].replace('\\', '/').lstrip('/')
                if os.path.islink(fullPath):
                    # http://www.mail-archive.com/python-list@python.org/msg34223.html
                    zipInfo = zipfile.ZipInfo(archiveRoot)
                    zipInfo.create_system = 3
                    # long type of hex val of '0xA1ED0000L',
                    # say, symlink attr magic...
                    #zipInfo.external_attr = 0777 << 16L
                    zipInfo.external_attr = '2716663808L'
                    zipOut.writestr(zipInfo, os.readlink(fullPath))
                else:
                    zipOut.write(fullPath, archiveRoot, zipfile.ZIP_DEFLATED)
    _ArchiveDirectory(inputDir)

    zipOut.close()


def get_struct_from_mp(formula, MAPI_KEY="", all_structs=False):
    """
    fetches the structure corresponding to the given formula
    from the materialsproject database.

    Note: Get the api key from materialsproject website. The one used
    here is nolonger valid.

    Note: for the given formula there are many structures available,
    this function returns the one with the lowest energy above the hull
    unless all_structs is set to True
    """
    if not MAPI_KEY:
        MAPI_KEY = os.environ.get("MAPI_KEY", "")
        if not MAPI_KEY:
            print('API key not provided')
            print('get API KEY from materialsproject and set it to the MAPI_KEY environment variable. aborting ... ')
            sys.exit()
    with MPRester(MAPI_KEY) as m:
        data = m.get_data(formula)
        structures = []
        x = {}
        print("\nnumber of structures matching the chemical formula {0} = {1}".format(formula, len(data)))
        print("The one with the the lowest energy above the hull is returned, unless all_structs is set to True")
        for d in data:
            mpid = str(d['material_id'])
            x[mpid] = d['e_above_hull']
            if all_structs:
                structure = m.get_structure_by_material_id(mpid)
                structure.sort()
                structures.append(structure)
        if all_structs:
            return structures
        else:
            mineah_key = sorted(x.items(), key=operator.itemgetter(1))[0][0]
            print("The id of the material corresponding to the lowest energy above the hull = {0}".format(mineah_key))
            if mineah_key:
                return mineah_key,m.get_structure_by_material_id(mineah_key)
            else:
                return None


def run_job(mat=None,parameters = {},jobname=''):
    """ 
    Generic  function for running LAMMPS job
    """
#def run_job(mat=None,parameters = {'exec':'/cluster/bin/lmp_ctcms-14439-knc6-2','pair_style':'comb3 polar_on','pair_coeff':None,'atom_style': 'charge' ,'control_file':'/home/kamal/inelast.mod'},jobname=''):
    jobname=str(mat.comment)
    #if jobname.startswith('bulk') or jobname.startswith('sbulk'):
    #   parameters['control_file']='/home/kamal/inelast.mod'
    #else:
    #   parameters['control_file']='/home/kamal/inelast_nobox.mod'
       
    folder=str(os.getcwd())+str("/")+str(jobname)
    if not os.path.exists(folder):
        os.makedirs(str(jobname))
    os.chdir(str(jobname))
    print ("folder name",folder) 
    forces='na'
    try:
       
       (en,press,toten,c11,c22,c33,c12,c13,c23,c44,c55,c66,c14,c16,c24,c25,c26,c34,c35,c36,c45,c46,c56)=analyz_loge('./log.lammps')
    except: 
        nprocs=1
        nnodes=1
        f=open('submit_job','w')
        line=str("#!/bin/bash")+'\n'
        f.write(line)
        line=str("#PBS -N ")+jobname+'\n'
        f.write(line)
        line=str("#PBS -l walltime=0:30:00")+'\n'
        f.write(line)
        line=str("#PBS -o test.log")+'\n'
        f.write(line)
        line=str("#PBS -m abe")+'\n'
        f.write(line)
        line=str("#PBS -j oe")+'\n'
        f.write(line)
        line=str("#PBS -r n")+'\n'
        f.write(line)
        line=str("#PBS -l nodes=")+str(nnodes)+str(":")+str("ppn=")+str(int(float(nprocs)/float(nnodes)))+'\n'
        f.write(line)
        #line=str("#PBS -l pmem=")+str(mem)+'\n'
        #f.write(line)
        dir=str(os.getcwd())
        line=str("cd ")+dir+'\n'
        f.write(line)
        cluster=parameters['cluster']
        job_bin=str(parameters['exec'])#str("mpirun /cluster/bin/lmp_ctcms-14439-knc6 <in.elastic >out")
        print ('job_bin',job_bin)
        line=str(job_bin)+'\n'
        f.write(line)
        f.close()
        write_lammps_data(structure=mat.structure, file='data')
        write_lammps_in(structure=mat.structure,lammps_in="init.mod",lammps_in1="potential.mod",lammps_in2="in.elastic", parameters = parameters)
        initial_str=read_data(data="data",ff="potential.mod")
        if cluster=='sun': #sungrid engine
        #if 'ruth' in socket.gethostname() or 'r049' in socket.gethostname():
           with open('job.out', 'w') as f:
                p = subprocess.Popen(['qsub','-cwd', '-pe', 'nodal', '1', 'submit_job'], stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE)
                stdout, stderr = p.communicate()
                job_id = str(stdout.split('Your job')[1].split(' ')[1])
                f.write(job_id)
           status = True
           while status !=False:
             try:
                line=str("qstat -j ")+str(job_id)
                a=os.system(line)
                if int(a) !=0:
                   status=False
                else:
                   time.sleep(5)
             except:
                   status=False
                   # print "whattyhjkl;'"
        elif cluster=='pbs':
           print ('cluster=',cluster)
        #elif 'dobby' in socket.gethostname():
           with open('job.out', 'w') as f:
                p = subprocess.Popen(['qsub', 'submit_job'], stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE)
                stdout, stderr = p.communicate()
                #self.job_id = stdout.rstrip('\n').split()[-1]
                print ("stdout,stderr",stdout, stderr)
                job_id = str(stdout.split('.')[0])#str(stdout.rstrip('\n').split()[-1])
                f.write(job_id)
           status = True
           while status !=False:
             #print ("status=",status, job_id)
             try:
                print ("qstat,jobid comeonnn",job_id)
                output = subprocess.check_output(['qstat', '-i', job_id])
                #state = output.rstrip('\n').split('\n')[-1].split()[-2]
                #time.sleep(5)
                print ('output',output)
                if 'Unknown' in output:
                #if 'C' in output or 'Unknown Job Id Error' in output:
                   status=False
             except:
                    status=False
                    print ("Error in job submission")
                 


        else:
           print ("socket error")

        time.sleep(100)
        (en,press,toten,c11,c22,c33,c12,c13,c23,c44,c55,c66,c14,c16,c24,c25,c26,c34,c35,c36,c45,c46,c56)=analyz_loge('./log.lammps')

    print ("initial,final sr",os.getcwd())
    initial_str=read_data(data="data",ff="potential.mod")
    final_str=read_data(data="data0",ff="potential.mod")
    try:
      forces=read_dump(data='0.dump',ff='potential.mod')
    except:
       pass
    print ("initial,final sr2",os.getcwd())

    data_cal=[]
    data_cal.append({'jobname':jobname,'poscar':mat.as_dict(),'initial_pos':initial_str.as_dict(),'pair_style':str(parameters['pair_style']),'pair_coeff':str(parameters['pair_coeff']),'final_energy':float(toten),'en':en,'press':press,'final_str':final_str.as_dict()})
    json_file=str(jobname)+str('.json')
    os.chdir('../')
    f_json=open(json_file,'w')
    f_json.write(json.dumps(data_cal,indent=4,cls= MontyEncoder))
    f_json.close()
    return en,final_str,forces






def write_lammps_data(structure=None, file=''):
        """
        write lammps structure data
        from ase with custom modifications
        """
        structure.sort()
        structure.to(fmt= "poscar", filename= "new_pymatgen_slab.vasp")
        atoms = AseAtomsAdaptor().get_atoms(structure)

        f=open(file,"w")
        f.write( 'datafile (written by JARVIS-FF) \n\n')
        symbols = atoms.get_chemical_symbols()
        import ase.io.vasp
        #ase.io.vasp.write_vasp("POSCAR.1x1x1",atoms, label='444supercell',direct=True,sort=False)
        n_atoms = len(symbols)
        f.write('%d \t atoms \n' % n_atoms)
        species = [tos for tos in Poscar(structure).site_symbols]
        #species = [tos.symbol
        #               for tos in structure.types_of_specie]
        n_atom_types = len(species)
        print ("species",species)
        f.write('%d  atom types\n' % n_atom_types)
        p = prism(atoms.get_cell())
        xhi, yhi, zhi, xy, xz, yz = p.get_lammps_prism_str()
        f.write('0.0 %s  xlo xhi\n' % xhi)
        f.write('0.0 %s  ylo yhi\n' % yhi)
        f.write('0.0 %s  zlo zhi\n' % zhi)
        f.write('%s %s %s  xy xz yz\n' % (xy, xz, yz))
        f.write('\n\n')
        f.write('Atoms \n\n')
        for i, r in enumerate(map(p.pos_to_lammps_str,
                                  atoms.get_positions())):
            c = 0.0
            s = species.index(symbols[i]) + 1
                    
            f.write('%6d %3d %6f %s %s %s\n' %
                            ((i+1, s, c)+tuple(r)) )
        f.close()


def write_lammps_in( structure=None,lammps_in=None,lammps_in1=None,lammps_in2=None, lammps_trj=None,
                        lammps_data=None,parameters={}):
        """
        write lammps input file
        from ase with custom modifications
        """     
        structure.sort()   
        f = open(lammps_in,"w")
        f1 = open(lammps_in1,"w") #potential.mod
        f2 = open(lammps_in2,"w")
        f.write(('variable dump_file string "%s"\n' % lammps_trj) +
        ('variable up  equal 1.0e-6\n' )+
        ('variable cfac  equal 1.0e-4\n' )+
        ('variable cunits  string GPa\n' )+
        ('variable etol  equal 0\n' )+
        ('variable ftol  equal 1.0e-10\n' )+
        ('variable maxiter equal 1000\n' )+
        ('variable maxeval equal 10000\n' )+
        ('variable dmax equal 1.0e-2\n' )+
        ('variable data_file string "%s"\n' % "data"))
        atoms = AseAtomsAdaptor().get_atoms(structure)
        pbc = atoms.get_pbc()
        if 'control_file' in parameters:
            f2.write('include %s \n'%  parameters['control_file'])
        if 'units' in parameters:
            f.write('units %s \n' % parameters['units'])
        else:
            f.write('units metal \n')
        if 'atom_style' in parameters:
            f.write('atom_style %s \n' % parameters['atom_style'])
        else:
            f.write('atom_style atomic \n')
        if 'boundary' in parameters:
            f.write('boundary %s \n' % parameters['boundary'])
        else:
            f.write('boundary %c %c %c \n' %
                    tuple('sp'[x] for x in pbc))
        f.write('atom_modify sort 0 0.0 \n')
        for key in ('neighbor' ,'newton'):
            if key in parameters:
                f.write('%s %s \n' % (key, parameters[key]))
        f.write('\n')
        # If no_data_file,
        # write the simulation box and the atoms
        species = [tos for tos in Poscar(structure).site_symbols]
        #species = [tos.symbol for tos in structure.types_of_specie]
        n_atom_types = len(species)
        species_i = dict([(s,i+1) for i,s in enumerate(species)])
        f.write('read_data %s\n' % "data")
        # interaction
        f.write('\n### interactions \n')
        if ( 'lib' in parameters ):
            lib = parameters['lib']
            f1.write('%s \n' % lib)
        if ( ('pair_style' in parameters) and ('pair_coeff' in parameters) ):
            pair_style = parameters['pair_style']
            f1.write('pair_style %s \n' % pair_style)
            symbols = atoms.get_chemical_symbols()
            #species = [tos.symbol.replace("Mo","M") for tos in structure.types_of_specie] #For REBO Mo-S
            #species = [tos.symbol for tos in structure.types_of_specie]
            print ("site symbolss",Poscar(structure).site_symbols)
            species = [tos for tos in Poscar(structure).site_symbols]
            if parameters['pair_style']=='rebomos':
                
                species = [tos.replace("Mo","M") for tos in Poscar(structure).site_symbols]
            tag=''
            for i in species:
                tag=tag+' '+i
            pair_coef =  '* * '+str(parameters['pair_coeff'])+' '+tag
            f1.write('pair_coeff %s \n' % pair_coef)

        

            masses=[]
            for m in  atoms.get_masses():
                if m not in masses:
                   masses.append(m)
            count=0
            for i in masses:
                count=count+1
                f.write('mass'+' '+str(count)+' '+str(i)+'\n')
        else:
            # default interaction
            f.write('pair_style lj/cut 2.5 \n' +
                    'pair_coeff * * 1 1 \n' +
                    'mass * 1.0 \n')
        f1.write('neighbor 1.0 nsq\n' )
        f1.write('neigh_modify once no every 1 delay 0 check yes\n' )
        if 'min' not in parameters: 
              f1.write('min_style  cg\n' )
              f1.write('min_modify           dmax ${dmax} line quadratic\n' )
        f1.write('thermo          1\n' )
        f1.write('thermo_style custom step temp press cpu pxx pyy pzz pxy pxz pyz ke pe etotal vol lx ly lz atoms\n' )
        #f1.write('thermo_style custom step temp pe press pxx pyy pzz pxy pxz pyz lx ly lz vol\n' )
        f1.write('thermo_modify norm no\n' )
     #   if 'thermo_style' in parameters:
     #       f.write('thermo_style %s\n' % parameters['thermo_style'])
     #   else:
     #       f.write(('thermo_style custom %s\n') %
     #               (' '.join(self._custom_thermo_args)))
     #   if 'thermo_modify' in parameters:
     #       f.write('thermo_modify %s\n' % parameters['thermo_modify'])
     #   else:
     #       f.write('thermo_modify flush yes\n')
     #   if 'thermo' in parameters:
     #       f.write('thermo %s\n' % parameters['thermo'])
     #   else:
     #       f.write('thermo 1\n')            
     #   if 'minimize' in parameters:
     #       f.write('minimize %s\n' % parameters['minimize'])
     #   if 'run' in parameters:
     #       f.write('run %s\n' % parameters['run'])
     #   if not (('minimize' in parameters) or ('run' in parameters)):
     #       f.write('run 0\n')
     #   if 'dump' in parameters:
     #       f.write('dump %s\n' % parameters['dump'])
     #   else:
     #       f.write('dump dump_all all custom 1 %s id type x y z vx vy vz fx fy fz\n' % lammps_trj)            
     #   f.write('print __end_of_ase_invoked_calculation__\n')
     #   f.write('log /dev/stdout\n')
        if 'fix' in parameters:
            if parameters['fix']:
                for i in parameters['fix']:
                    f1.write('fix %s\n' % i)
        f.close()
        f1.close()
        f2.close()


def analyz_loge(log='log.lammps'):
    import sys
    """ 
    Analyzes log.lammps file, at present  energy/atom data extraction is implemented
    """
    en=0
    press=0
    c11=0
    c22=0
    c33=0
    c44=0
    c55=0
    c66=0
    c12=0
    c13=0
    c23=0
    c14=0
    c15=0
    c16=0
    c14=0
    c24=0
    c25=0
    c26=0
    c34=0
    c35=0
    c36=0
    c45=0
    c46=0
    c56=0
    try:
       logfile = open(log, "r")
       lines = logfile.read().splitlines()
       for i, line in enumerate(lines):
           if 'Loop time of' in line:
               toten=float(lines[i-1].split()[12])
               press=float(lines[i-1].split()[2])
               press=float(press)*0.0001
               en=float(lines[i-1].split()[12])/float(lines[i-1].split()[17])
               break
       logfile.close()
    except:
       pass
    try:
       logfile = open(log, "r")
       lines = logfile.read().splitlines()
       for i, line in enumerate(lines):
           if 'print "Elastic Constant C11all = ${C11all} ${cunits}"' in line:
               c11=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C22all = ${C22all} ${cunits}"' in line:
               c22=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C33all = ${C33all} ${cunits}"' in line:
               c33=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C12all = ${C12all} ${cunits}"' in line:
               c12=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C13all = ${C13all} ${cunits}"' in line:
               c13=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C23all = ${C23all} ${cunits}"' in line:
               c23=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C44all = ${C44all} ${cunits}"' in line:
               c44=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C55all = ${C55all} ${cunits}"' in line:
               c55=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C66all = ${C66all} ${cunits}"' in line:
               c66=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C14all = ${C14all} ${cunits}"' in line:
               c14=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C16all = ${C16all} ${cunits}"' in line:
               c16=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C24all = ${C24all} ${cunits}"' in line:
               c24=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C25all = ${C25all} ${cunits}"' in line:
               c25=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C26all = ${C26all} ${cunits}"' in line:
               c26=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C34all = ${C34all} ${cunits}"' in line:
               c34=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C35all = ${C35all} ${cunits}"' in line:
               c35=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C36all = ${C36all} ${cunits}"' in line:
               c36=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C45all = ${C45all} ${cunits}"' in line:
               c45=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C46all = ${C46all} ${cunits}"' in line:
               c46=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
           if 'print "Elastic Constant C56all = ${C56all} ${cunits}"' in line:
               c56=((str((lines[i+1])).split("=")[1]).split("GPa"))[0]
       logfile.close()
    except:
       pass
    return round(en,2),round(press,2),float(toten),round(float(c11),1),round(float(c22),1),round(float(c33),1),round(float(c12),1),round(float(c13),1),round(float(c23),1),round(float(c44),1),round(float(c55),1),round(float(c66),1),round(float(c14),1),round(float(c16),1),round(float(c24),1),round(float(c25),1),round(float(c26),1),round(float(c34),1),round(float(c35),1),round(float(c36),1),round(float(c45),1),round(float(c46),1),round(float(c56),1)
def read_dumpfull(data=None,ff=None):
    pot_file=open(ff,"r")
    lines = pot_file.read().splitlines()
    symb=[]
    count=0
    from pymatgen.core.periodic_table import Element
    for i, line in enumerate(lines):
        if "pair_coeff" in line.split():
               sp=line.split()
               for el in sp:
                   try:
                    if Element(el):
                     #if el=='M':
                   #    el='Mo'
                   #count=count+1
                   #if count >4:
                      symb.append(el)
                   except:
                      pass
    print ("symb=",symb)

    f=open(data,"r")
    lines = f.read().splitlines()
    for i, line in enumerate(lines):
        if "NUMBER OF ATOMS" in line:
             natoms=int(lines[i+1].split()[0])
        if "ITEM: BOX BOUNDS" in line:
             xlo=float(lines[i+1].split()[0])
             xhi=float(lines[i+1].split()[1])
             ylo=float(lines[i+2].split()[0])
             yhi=float(lines[i+2].split()[1])
             zlo=float(lines[i+3].split()[0])
             zhi=float(lines[i+3].split()[1])
    xy=0
    yz=0
    xz=0
    x=np.zeros((natoms))
    y=np.zeros((natoms))
    z=np.zeros((natoms))
    lat=Lattice([[ xhi-xlo,0.0,0.0],[xy,yhi-ylo,0.0],[xz,yz,zhi-zlo]])
    q=np.zeros((natoms))
    typ= np.empty((natoms),dtype="S20")
    coords = list()#np.zeros((natoms))
    for i, line in enumerate(lines):
        if "ITEM: ATOMS" in line:
           for j in range(0,natoms):
                 typ[j]=symb[int(((lines[i+j+1]).split()[1]))-1]
                 #typ[j]=(lines[i+j+1]).split()[1]
                 x[j]=(lines[i+j+1]).split()[3]
                 y[j]=(lines[i+j+1]).split()[4]
                 z[j]=(lines[i+j+1]).split()[5]
                 coords.append([x[j],y[j],z[j]])
    f.close()
    pot_file.close()
    struct=Structure(lat,typ,coords,coords_are_cartesian=True)
    return struct

def read_dump(data=None,ff=None):
    """ 
    Read LAMMPS dump file
    """
    pot_file=open(ff,"r")
    lines = pot_file.read().splitlines()
    symb=[]
    count=0
    from pymatgen.core.periodic_table import Element
    for i, line in enumerate(lines):
        if "pair_coeff" in line.split():
               sp=line.split()
               print ("spsplit",sp,os.getcwd())
               for el in sp:
                   try:
                    if Element(el):
                     #if el=='M':
                   #    el='Mo'
                   #count=count+1
                   #if count >4:
                      symb.append(el)
                   except:
                      pass
    print ("symb=",symb)

    f=open(data,"r")
    lines = f.read().splitlines()
    for i, line in enumerate(lines):
        if "NUMBER OF ATOMS" in line:
             natoms=int(lines[i+1].split()[0])
    x=np.zeros((natoms))
    y=np.zeros((natoms))
    z=np.zeros((natoms))
    coords = list()#np.zeros((natoms))
    for i, line in enumerate(lines):
        if "ITEM: ATOMS" in line:
           for j in range(0,natoms):
                 x[j]=(lines[i+j+1]).split()[1]
                 y[j]=(lines[i+j+1]).split()[2]
                 z[j]=(lines[i+j+1]).split()[3]
                 coords.append([x[j],y[j],z[j]])
    f.close()
    pot_file.close()
    prop=np.asarray(coords)
    return prop

def read_data(data=None,ff=None):
    """
    Read LAMMPS data file
    """
    pot_file=open(ff,"r")
    lines = pot_file.read().splitlines()
    symb=[]
    count=0
    from pymatgen.core.periodic_table import Element
    for i, line in enumerate(lines):
        if "pair_coeff" in line.split():
               sp=line.split()
               print ("spsplit",sp,os.getcwd())
               for el in sp:
                   try:
                    if Element(el):
                     #if el=='M':
                   #    el='Mo'
                   #count=count+1
                   #if count >4:
                      symb.append(el)
                   except:
                      pass
    print ("symb=",symb) 
            
    f=open(data,"r")
    lines = f.read().splitlines()
    for i, line in enumerate(lines):
        if "atoms" in line.split():
             natoms=int(line.split()[0])
        if "types" in line.split():
             print (line)
             ntypes=int(line.split()[0])
        if "xlo" in line.split():
             xlo=float(line.split()[0])
             xhi=float(line.split()[1])
        if "ylo" in line.split():
             ylo=float(line.split()[0])
             yhi=float(line.split()[1])
        if "zlo" in line.split():
             zlo=float(line.split()[0])
             zhi=float(line.split()[1])
        if "xy" in line.split():
             xy=float(line.split()[0])
             xz=float(line.split()[1])
             yz=float(line.split()[2])
    if len(symb) != ntypes:
        print ("Something wrong in atom type assignment",len(symb),ntypes)
        sys.exit()
    lat=Lattice([[ xhi-xlo,0.0,0.0],[xy,yhi-ylo,0.0],[xz,yz,zhi-zlo]])
    typ= np.empty((natoms),dtype="S20")
    x=np.zeros((natoms))
    y=np.zeros((natoms))
    z=np.zeros((natoms))
    q=np.zeros((natoms))
    coords = list()
    for i, line in enumerate(lines):
        if "Atoms" in line.split():
           for j in range(0,natoms):
                 #print int(((lines[j+2]).split()[1]))-1
                 typ[j]=symb[int(((lines[i+j+2]).split()[1]))-1]
                 q[j]=(lines[i+j+2]).split()[2]
                 x[j]=(lines[i+j+2]).split()[3]
                 y[j]=(lines[i+j+2]).split()[4]
                 z[j]=(lines[i+j+2]).split()[5]
                 coords.append([x[j],y[j],z[j]])
    f.close()
    print ("info",len(typ),len(coords))
    pot_file.close()
    struct=Structure(lat,typ,coords,coords_are_cartesian=True)
    #print struct
    finder = SpacegroupAnalyzer(struct)
    num=finder.get_spacegroup_symbol()
    #print(num)
    return struct
def smart_vac(strt=None,parameters=None):
    """
    Function to get all vacancy formation energies
    """
    parameters['control_file']='/users/knc6/inelast_nobox.mod'
    vac_arr=[]
    sg_mat = SpacegroupAnalyzer(strt)
    mat_cvn = sg_mat.get_conventional_standard_structure()
    mat_cvn.sort()

    cellmax=2 #int(mat_cvn.composition.num_atoms)+int(mat_cvn.ntypesp)#5
    ase_atoms = AseAtomsAdaptor().get_atoms(mat_cvn)
    ase_atoms=ase_atoms*(cellmax,cellmax,cellmax)
    #if len(ase_atoms) >200:
    #   cellmax=1
    #else:
    #   cellmax=2
    #cellmax=int(mat_cvn.composition.num_atoms)+int(mat_cvn.ntypesp)#5
    print ("type of trt is= celmmax",type(strt),cellmax)
    try:
       print ("int(strt.composition.num_atoms)",int(strt.composition.num_atoms))
       print (int(strt.ntypesp))
    except:
       pass
    #cellmax=int(strt.composition.num_atoms)+int(strt.ntypesp)
    vac_done=0
    tol=0.01   #change 0.1
    try:
        vac=vac_antisite_def_struct_gen(cellmax=cellmax,struct=strt)
    except:
        print ("Failed for v_1",os.getcwd())
        pass
    def_list=[100000  for y in range(len(vac)-1)]
    while vac_done !=1:
        try:
            vac=vac_antisite_def_struct_gen(cellmax=cellmax,struct=strt)
        except:
            print ("Failed for v_2",os.getcwd())
            pass
        if vac not in vac_arr:
           try:
               vac_arr.append(vac)
               print ("in smart_vac(strt=None), cellmax,vac,vac_done=",cellmax,vac,vac_done)
               def_list2,header_list=def_energy(vac=vac,parameters=parameters)
               diff=matrix(def_list)-matrix(def_list2)
               diff_arr=np.array(diff).flatten()
               print ("in smart_vac(strt=None diff_arr=",diff_arr)
               print ("sleepig for 5")
               #time.sleep(5)
           except:
                print ("Failed for v_3",os.getcwd())
                pass
           if any(diff_arr)>tol:
          # for el in diff_arr:
          #     if abs(el)>tol :
                  # print ("in smart_vac(strt=None abs_el=",abs(el))
                   vac_done=0
                   cellmax=cellmax+1
                   ase_atoms = AseAtomsAdaptor().get_atoms(mat_cvn)
                   ase_atoms=ase_atoms*(cellmax,cellmax,cellmax)
                   if len(ase_atoms) >50:
                      vac_done=1
                      break
                   def_list=def_list2
           else:
               vac_done=1
#        cellmax=cellmax+1
    return def_list,header_list

def smart_surf(strt=None,parameters=None):
    """
    Function to get all surface energies
    """
    parameters['control_file']='/users/knc6/inelast_nobox.mod'
    sg_mat = SpacegroupAnalyzer(strt)
    mat_cvn = sg_mat.get_conventional_standard_structure()
    mat_cvn.sort()
    layers=3
    indices = get_symmetrically_distinct_miller_indices(mat_cvn, 1)
    ase_atoms = AseAtomsAdaptor().get_atoms(mat_cvn)
    for i in indices:
        ase_slab = surface(ase_atoms, i, layers)
        ase_slab.center(vacuum=15, axis=2)
        if len(ase_slab) < 50:
           layers=3
    surf_arr=[]
    surf_done=True
    tol=0.5 #change 0.01
    try:
        surf=surfer(mat=strt,layers=layers)
        surf_list=[100000  for y in range(len(surf)-1)]
        print ("in smart_surf :surf,surf_list=",surf, surf_list)
    except:
       print ("Failed at s1",os.getcwd())
       pass
    while surf_done :
        layers=layers+1
        indices = get_symmetrically_distinct_miller_indices(mat_cvn, 1)
        ase_atoms = AseAtomsAdaptor().get_atoms(mat_cvn)
        for i in indices:
            ase_slab = surface(ase_atoms, i, layers)
            ase_slab.center(vacuum=15, axis=2)
            #if len(ase_slab) > 100:
            #   surf_done=True
            #if (ase_slab.get_cell()[2][2]) > 40:
            #   surf_done=True
           

        try:  
           surf=surfer(mat=strt,layers=layers)
        except:
           
           print ("Failed at s2",os.getcwd())
           pass
        if surf not in surf_arr:
           surf_arr.append(surf)
           try:
               surf_list2,surf_header_list=surf_energy(surf=surf,parameters=parameters)
               print ("in smart_surf :surf2,surf_list2=",surf_list2,surf_header_list)
               diff=matrix(surf_list)-matrix(surf_list2)
               print ("in smart_surf :surf3,surf_list3=",matrix(surf_list),matrix(surf_list2))
               diff_arr=np.array(diff).flatten()
           except:
               print ("Failed for s_3",os.getcwd())
               pass
           if len(ase_slab) >50:
                      surf_done=True
                      break
          # print ("layersssssssssssssssssssssssss",layers,surf_done)
                      break
           if any(diff_arr)>tol:
           #for el in diff_arr:
           #    if abs(el)>tol :
           #        print ("in smart_surf :abs el=",abs(el))
                   surf_done=True
                   surf_list=surf_list2
           else:
                surf_done=False
    return surf_list,surf_header_list

def surf_energy(surf=[],parameters = {}):
    """
    Function to get specific surface energies
    """
#def surf_energy(surf=[],parameters = {'pair_style':'eam/alloy','pair_coeff':'/scratch/lfs/kamal/POSMAT/Automatic2/Al03.eam.alloy','atom_style': 'charge' ,'control_file':'/home/kamal/inelast.mod'}):
    surf_list=[]
    surf_header_list=[]
    fi=str('SURF.INFO')
    f=open(fi,'w')
    for s in surf:
        enp,strt,forces=run_job(mat=s,parameters = parameters)
        #strt=Structure.from_file(contc)
        m=strt.lattice.matrix
        if s.comment.split('@')[0]=='sbulk':

           gs_energy=enp
           print ("in surf_energy gs_energy for",s.comment, gs_energy)
        else:
           surf_en=16*(float(enp)*(strt.composition.num_atoms)-float(strt.composition.num_atoms)*float(gs_energy))/(2*np.linalg.norm(np.cross(m[0], m[1])))
           print ("in surf_energy surf_en for",s.comment, surf_en)
           surf_list.append(surf_en)
           print (s.comment,'=',surf_en)
           line= str(s.comment)+str('=')+str(surf_en)+'\n'
           f.write(line)
    f.close()
    return surf_list,surf_header_list

def get_chem_pot(s1=None,s2=None,parameters= {}):
    """
    Get chemical potential given perfect and defect structures
    """
#def get_chem_pot(s1=None,s2=None,parameters= {'pair_style':'eam/alloy','pair_coeff':'/scratch/lfs/kamal/POSMAT/Automatic2/Al03.eam.alloy','atom_style': 'charge' ,'control_file':'/home/kamal/inelast.mod'}):
    s1.sort() 
    s2.sort() 
    s3=(set(s2)).symmetric_difference(set(s1))#list(set(s1)-set(s2))
    #s3=[]
    #for el1 in s1:
    #     for el2 in s2:
    #         if el1 !=el2 and el1 not in s3:
    #            s3.append(el1)
    #         if el1 !=el2 and el2 not in s3:
    #            s3.append(el2)
    #             
    #from pymatgen.analysis.structure_matcher import StructureMatcher
    #print "Mather",StructureMatcher().fit(s1, s2)
    print ("s3   is   ",type(s1),type(s2),s3)
    uniq=[]
    for q in s3:
        el=q._species.elements 
        for j in el:
           print   ("j is ",j)
           if j not in uniq:   
              uniq.append(j)
              a,b= get_struct_from_mp(j)
              p=Poscar(b)
              p.comment=str(a)
              enp,strt,forces=run_job(mat=p,parameters = parameters)
    if len(uniq)>1:
        print ("uniq problem",uniq)
    return enp

def calc_forces(mat=None,parameters={}):
    """
    Calculate forces on atoms
    """
    enp,strt,forces=run_job(mat=mat,parameters = parameters)
    return forces

def do_phonons(strt=None,parameters=None):
    """
    Setting up phonopy job
    """
         
    p= get_phonopy_atoms(mat=strt)
    bulk =p
    c_size=25
    dim1=int((float(c_size)/float( max(abs(strt.lattice.matrix[0])))))+1
    dim2=int(float(c_size)/float( max(abs(strt.lattice.matrix[1]))))+1
    dim3=int(float(c_size)/float( max(abs(strt.lattice.matrix[2]))))+1
    Poscar(strt).write_file("POSCAR")
    tmp=strt.copy()
    tmp.make_supercell([dim1,dim2,dim3])
    Poscar(tmp).write_file("POSCAR-Super.vasp")
    
    phonon = Phonopy(bulk,[[dim1,0,0],[0,dim2,0],[0,0,dim3]])#,
    print ("[Phonopy] Atomic displacements:")
    disps = phonon.get_displacements()
    for d in disps:
      print ("[Phonopy]", d[0], d[1:])
    supercells = phonon.get_supercells_with_displacements()

    # Force calculations by calculator
    set_of_forces = []
    disp=0
    for scell in supercells:
        cell = Atoms(symbols=scell.get_chemical_symbols(),
			 scaled_positions=scell.get_scaled_positions(),
			 cell=scell.get_cell(),
			 pbc=True)
        disp=disp+1

    mat = Poscar(AseAtomsAdaptor().get_structure(cell))
    mat.comment=str("disp-")+str(disp)
    parameters['min']='skip'
    parameters['control_file']='/users/knc6/in.phonon'
    #a,b,forces=run_job(mat=mat,parameters={'min':'skip','pair_coeff': '/data/knc6/JARVIS-FF-NEW/ALLOY4/Mishin-Ni-Al-2009.eam.alloy', 'control_file': '/users/knc6/in.phonon', 'pair_style': 'eam/alloy', 'atom_style': 'charge'})
    a,b,forces=run_job(mat=mat,parameters=parameters)
    print ("forces=",forces)
    drift_force = forces.sum(axis=0)
    print ("drift forces=",drift_force)
    print ("[Phonopy] Drift force:", "%11.5f"*3 % tuple(drift_force))
    # Simple translational invariance
    for force in forces:
          force -= drift_force / forces.shape[0]
    set_of_forces.append(forces)
    phonon.produce_force_constants(forces=set_of_forces)

    write_FORCE_CONSTANTS(phonon.get_force_constants(),filename="FORCE_CONSTANTS")
    print ()
    print ("[Phonopy] Phonon frequencies at Gamma:")
	#for i, freq in enumerate(phonon.get_frequencies((0, 0, 0))):
	#    print ("[Phonopy] %3d: %10.5f THz" %  (i + 1, freq) # THz)











def def_energy(vac=[],parameters={}):
    """
    Get specific defect formation energy
    """
#def def_energy(vac=[],parameters={'pair_style':'eam/alloy','pair_coeff':'/scratch/lfs/kamal/POSMAT/Automatic2/Al03.eam.alloy','atom_style': 'charge' ,'control_file':'/home/kamal/inelast.mod'}):
    def_list=[]
    header_list=[]
    fi=str('DEF.INFO')
    f=open(fi,'w')
    for v in vac:
        enp,strt,forces=run_job(mat=v,parameters = parameters)
        #enp,contc=run_job(mat=v,incar=incar,kpoints=kpoints,jobname=str(v.comment))
        #strt=Structure.from_file(contc)
        comm=str(v.comment)
        print ("running def_energy for =",comm)
        header_list.append(comm)
        if comm.split('@')[0]=='bulk':
           gs_energy=float(enp)*float(strt.composition.num_atoms)
           gs_str=strt
           print ("in def_energy gs_energy for",comm, gs_energy)
        else:
           print ('strt',strt)
           print ('gs_str',gs_str)
           chem_pot=get_chem_pot(gs_str,strt,parameters = parameters)
           if comm.startswith("intl"):
              chem_pot=0.0-float(chem_pot)
           def_en=(float(enp)*float(strt.composition.num_atoms))-float(gs_energy)+chem_pot
           print ("in def_energy def_en for",comm, def_en,"chem_pot",chem_pot)
           def_list.append(def_en)
           print (v.comment,'=',def_en)
           line= str(v.comment)+str('=')+str(def_en)+'\n'
           f.write(line)
    f.close()
    return def_list,header_list

#def main(p=None, parameters={'pair_style':'rebomos','pair_coeff':'/scratch/lfs/kamal/JARVIS/All2/MoS2/MoS.REBO.set5b','atom_style': 'charge' ,'control_file':'/home/kamal/inelast.mod'}):
def main(p=None, parameters={}):
    """
    Master function to run LAMMPS job
    """
    #p=Poscar.from_file("POSCAR")
    c_size=35
    sg_mat = SpacegroupAnalyzer(p.structure)
    mat_cvn = sg_mat.get_conventional_standard_structure()
    dim1=int((float(c_size)/float( max(abs(mat_cvn.lattice.matrix[0])))))+1
    dim2=int(float(c_size)/float( max(abs(mat_cvn.lattice.matrix[1]))))+1
    dim3=int(float(c_size)/float( max(abs(mat_cvn.lattice.matrix[2]))))+1
    cellmax=max(dim1,dim2,dim3)
    #print "dim1 dim2 dim3",dim1,dim2,dim3
    #mat_cvn.make_supercell([dim1,dim2,dim3])
    mat_pos=Poscar(mat_cvn)
    mat_pos.comment=str(p.comment)
    print (mat_pos)
    try:
       toten,final_str,forces=run_job(mat=mat_pos,parameters = parameters)
    except:
       pass
    print ("p.comment issssss",p.comment)
    vac=vac_antisite_def_struct_gen(c_size=c_size,struct=final_str)
    def_list,header_list=def_energy(vac=vac,parameters = parameters)
    print (def_list,header_list)
    try:
       print ("p.comment issssss",p.comment)
       vac=vac_antisite_def_struct_gen(c_size=c_size,struct=final_str)
       def_list,header_list=def_energy(vac=vac,parameters = parameters)
       print (def_list,header_list)
    except:
       pass
    try:
        surf=pmg_surfer(mat=final_str, min_slab_size=35,vacuum=35,max_index=3)
        surf_list,surf_header_list=surf_energy(surf=surf,parameters = parameters)
        print (surf_list,surf_header_list)
    except:
       pass
    try:
       cwd=str(os.getcwd())
       if not os.path.exists("Phonon"):
          os.mkdir("Phonon")
       os.chdir("Phonon")
       do_phonons(strt=final_str,parameters=parameters)
       os.chdir(cwd)
    except:
       pass
    sub_files=[]
    calc=0
    for a in glob.glob("*.json"):
        fold=a.split(".json")[0]
        cwd=os.getcwd()
        target_file=str(cwd)+str("/")+str(fold)
        dest_file=str(cwd)+str("/")+str(fold)+str(".zip")
        sub_files.append(dest_file)
        ZipDir(target_file,dest_file,contents=['init.mod', 'potential.mod', 'in.elastic', 'data',  'log.lammps', 'restart.equil','data0' ])
    calc=calc+1

    target_file=str(cwd)
    dest_file=str(cwd)+str("/")+str("Calc-")+str(calc)+str(".zip")
    ZipDir(target_file,dest_file,contents=sub_files)
    for a in glob.glob("*.json"):
        fold=a.split(".json")[0]
        cwd=os.getcwd()
        dest_file=str(cwd)+str("/")+str(fold)+str(".zip")
        line=str("rm ")+str(dest_file)
        os.system(line)

def main_func(mpid='',mat=None,parameters={}):
    """
    Call master job function either using mpid or Poscar object
    """
    if mpid !='':
       with MPRester() as mp:
         strt = mp.get_structure_by_material_id(mpid)
         mat=Poscar(strt)

         mpid=mpid.replace('-','_')
         mpid=str('bulk@')+str(mpid)
         mat.comment=mpid

    main(p=mat,parameters=parameters)
#Example
def main_alloy():
   """
   Run alloy FFs job
   """
   for file in glob.glob("*.alloy"):
    try:
        folder1=str(os.getcwd())+str("/")+str(file)+str("_nist")
        if not os.path.exists(folder1):
           os.makedirs(str(folder1))
        cwd1=str(os.getcwd())
        print ("folder1=",folder1)
        ff=str(file)
        element_ff=[]
        f=open(ff,'r')
        os.chdir(str(folder1))
        list_el=[]
        lines=f.readlines()
        content=(lines[3]).split(" ")
    #content=(lines[3]).split("' '|\n|\r\n")
        for val in content:
        
            if val != '' and val !='\n' and val !='\r\n':
               list_el.append(val)
        for i in range(0,len(list_el)):
             if i!=0:
                 element_ff.append(list_el[i])    
#    print ff,' ',element_ff
        with MPRester(MAPI_KEY) as m:
             data = m.get_entries_in_chemsys(element_ff,inc_structure='final', property_data=["unit_cell_formula","material_id","icsd_id","spacegroup","energy_per_atom","formation_energy_per_atom","pretty_formula","band_gap","total_magnetization","e_above_hull"])
             if (len(element_ff)>1):
                 try:
                     entries = m.get_entries_in_chemsys(element_ff)
                     pd = PhaseDiagram(entries)
                     plotter = PDPlotter(pd, show_unstable=True)
                     image=str(ff)+str("_DFT")+str(".jpg")
                     plotter.write_image(image)
                 except:
                     pass
             structures=[]
             structures_cvn=[]
             icsd_arr=[]
             mp_arr=[]
             sg_arr=[]
             enp_arr=[]
             fenp_arr=[]
             pf_arr=[]
             ucf_arr=[]
             bg_arr=[]
             tm_arr=[]
             ehull_arr=[]
             for d in data:
                 x=d.data["material_id"]
                 sg=d.data["spacegroup"]
                 enp=d.data["energy_per_atom"]
                 fenp=d.data["formation_energy_per_atom"]
                 pf=d.data["pretty_formula"]
                 ucf=d.data["unit_cell_formula"]
                 bg=d.data["band_gap"]
                 tm=d.data["total_magnetization"]
                 ehull=d.data["e_above_hull"]
                 icsd=d.data["icsd_id"]
                 structure = m.get_structure_by_material_id(x)
                 structures.append(structure)
                 icsd_arr.append(icsd)
                 mp_arr.append(x)
                 sg_arr.append(sg)
                 enp_arr.append(enp)
                 fenp_arr.append(fenp)
                 pf_arr.append(pf)
                 bg_arr.append(bg)
                 tm_arr.append(tm)
                 ucf_arr.append(ucf)
                 ehull_arr.append(ehull)
              

                 comment=str("bulk@")+str(x)
                 folder2=str(os.getcwd())+str("/")+str(comment)+str("_fold")
                 if not os.path.exists(folder2):
                     os.makedirs(str(folder2))
                 print ("folder2=",folder2)
                 cwd2=str(os.getcwd())
                 os.chdir(str(folder2))

                 p=Poscar(structure)
                 p.comment=comment
                 p.write_file("POSCAR")
                 poscar_file=str(os.getcwd())+str("/POSCAR")

                 pair_coeff=str(cwd1)+str("/")+str(file)
                 #pair_coeff=str('/data/knc6/JARVIS-FF-NEW/ALLOY')+str("/")+str(file)
                 parameters = {'pair_style':'eam/alloy','pair_coeff':pair_coeff,'atom_style': 'charge' ,'control_file':'/users/knc6/inelast.mod'}
                 main_file=open("setup.py","w")
                 line=str("from jlammps import main_func")+'\n'
                 main_file.write(line)
                 line=str("from pymatgen.io.vasp.inputs import  Poscar")+'\n'
                 main_file.write(line)
                 #line=str("try:")+'\n'
                 #main_file.write(line)
                 line=str("p=Poscar.from_file(")+str('"')+str(poscar_file)+str('"')+str(")")+'\n'
                 main_file.write(line)
                 line=str("main_func(mat=p")+str(",")+str("parameters=")+str(parameters)+str(")")+'\n'
                 main_file.write(line)
                 #line=str("except:")+'\n'
                 #main_file.write(line)
                 #line=str("     pass")+'\n'
                 #main_file.write(line)
                 main_file.close()
                 #try:
                 #    p=Poscar.from_file(poscar_file)
                 #    main_func(mat=p,parameters=parameters)
                 #except:
                 #     pass
                 os.chdir(cwd2)#=str(os.getcwd())
        os.chdir(cwd1)#=str(os.getcwd())
    except:
       pass



from __future__ import division, unicode_literals

"""
A script with tools for computing point defect concentrations.
Manual and citation for the script, DOI: 10.1016/j.cpc.2015.03.015
"""


import argparse
import os
from pymatgen.core.structure import Structure
from pymatgen.core.periodic_table import Element
from pymatgen.io.vasp import Poscar
from pymatgen.ext.matproj import MPRester
from pymatgen.analysis.defects.core import Vacancy,Interstitial
from pymatgen.analysis.local_env import ValenceIonicRadiusEvaluator
#from pymatgen.analysis.defects.point_defects import Vacancy,Interstitial,ValenceIonicRadiusEvaluator
from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
import glob
from pymatgen.io.vasp import Poscar
from monty.serialization import loadfn, dumpfn
from monty.json import MontyEncoder, MontyDecoder
#from pymatgen.analysis.defects.point_defects import Vacancy
#from pymatgen.io.vasp.sets import MPGGAVaspInputSet
from pymatgen.core.surface import Slab, SlabGenerator
from pymatgen.core.structure import Structure
from pymatgen.io.ase import AseAtomsAdaptor
from ase.lattice.surface import surface
from pymatgen.core.surface import  Slab, SlabGenerator, generate_all_slabs,get_symmetrically_distinct_miller_indices
from pymatgen.io.vasp import Kpoints
from pymatgen.io.vasp import Vasprun
from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
from pymatgen.analysis.defects.dilute_solution_model import \
            compute_defect_density, solute_defect_density



def get_sc_scale(inp_struct, final_site_no):
    lengths = inp_struct.lattice.abc
    no_sites = inp_struct.num_sites
    mult = (final_site_no/no_sites*lengths[0]*lengths[1]*lengths[2]) ** (1/3)
    num_mult = [int(round(mult/l)) for l in lengths]
    num_mult = [i if i > 0 else 1 for i in num_mult]
    return num_mult


def vac_antisite_def_struct_gen(c_size=15,mpid='',struct=None):
    def_str=[]
    if struct ==None:
        with MPRester() as mp:
                struct = mp.get_structure_by_material_id(mpid)
        if mpid == '':
           print ("Provide structure")
    c_size=c_size
    dim1=int((float(c_size)/float( max(abs(struct.lattice.matrix[0])))))+1
    dim2=int(float(c_size)/float( max(abs(struct.lattice.matrix[1]))))+1
    dim3=int(float(c_size)/float( max(abs(struct.lattice.matrix[2]))))+1
    cellmax=max(dim1,dim2,dim3)
    #print ("in vac_def cellmax=",cell
    prim_struct_sites = len(struct.sites)
    struct = SpacegroupAnalyzer(struct).get_conventional_standard_structure()
    conv_struct_sites = len(struct.sites)
    conv_prim_rat = int(conv_struct_sites/prim_struct_sites)
    #sc_scale = get_sc_scale(struct,cellmax)
    sc_scale=[dim1,dim2,dim3]
    #sc_scale=[cellmax,cellmax,cellmax]
    print ("sc_scale",sc_scale)
    #mpvis = MPGGAVaspInputSet()

    # Begin defaults: All default settings.
    #blk_vasp_incar_param = {'IBRION':-1,'EDIFF':1e-4,'EDIFFG':0.001,'NSW':0,}
    #def_vasp_incar_param = {'ISIF':2,'NELM':99,'IBRION':2,'EDIFF':1e-6, 
    #                        'EDIFFG':0.001,'NSW':40,}
    #kpoint_den = 6000
    # End defaults
    
    #ptcr_flag = True
    #try:
    #    potcar = mpvis.get_potcar(struct)
    #except:
    #    print ("VASP POTCAR folder not detected.\n" \
    #          "Only INCAR, POSCAR, KPOINTS are generated.\n" \
    #          "If you have VASP installed on this system, \n" \
    #          "refer to pymatgen documentation for configuring the settings.")
    #    ptcr_flag = False





    struct_valrad_eval = ValenceIonicRadiusEvaluator(struct)
    val = struct_valrad_eval.valences
    rad = struct_valrad_eval.radii
    struct_val = val
    struct_rad = rad




    vac = Vacancy(struct, {}, {})
    scs = vac.make_supercells_with_defects(sc_scale)
    #site_no = scs[0].num_sites
    #if site_no > cellmax:
    #    max_sc_dim = max(sc_scale)
    #    i = sc_scale.index(max_sc_dim)
    #    sc_scale[i] -= 1
    #    scs = vac.make_supercells_with_defects(sc_scale)


    for i in range(len(scs)):
        sc = scs[i]
        poscar = mpvis.get_poscar(sc)
        kpoints = Kpoints.automatic_density(sc,kpoint_den)
        incar = mpvis.get_incar(sc)
        #if ptcr_flag:
        #    potcar = mpvis.get_potcar(sc)

        interdir = mpid
        if not i:
            fin_dir = os.path.join(interdir,'bulk')
            #try:
            #    os.makedirs(fin_dir)
            #except:
            #    pass
            #incar.update(blk_vasp_incar_param)
            #incar.write_file(os.path.join(fin_dir,'INCAR'))
            #poscar.write_file(os.path.join(fin_dir,'POSCAR'))
            poscar.comment=str('bulk')+str('@')+str('cellmax')+str(cellmax)
            def_str.append(poscar)
            poscar.write_file('POSCAR-'+str('bulk')+str(".vasp"))
            #if ptcr_flag:
            #    potcar.write_file(os.path.join(fin_dir,'POTCAR'))
            #kpoints.write_file(os.path.join(fin_dir,'KPOINTS'))
        else:
            blk_str_sites = set(scs[0].sites)
            vac_str_sites = set(sc.sites)
            vac_sites = blk_str_sites - vac_str_sites
            vac_site = list(vac_sites)[0]
            site_mult = int(vac.get_defectsite_multiplicity(i-1)/conv_prim_rat)
            #try:
            #   site_mult = int(vac.get_defectsite_multiplicity(i-1)/conv_prim_rat)
            #except:
            #   site_mult=1
            #   pass
            vac_site_specie = vac_site.specie
            vac_symbol = vac_site.specie.symbol

            vac_dir ='vacancy_{}_mult-{}_sitespecie-{}'.format(str(i),
                    site_mult, vac_symbol)
            fin_dir = os.path.join(interdir,vac_dir)
            #try:
            #    os.makedirs(fin_dir)
            #except:
            #    pass
            #incar.update(def_vasp_incar_param)
            try:
                poscar.comment=str(vac_dir)+str('@')+str('cellmax')+str(cellmax)
            except:
                pass
            pos=poscar
            #pos=poscar.structure
            def_str.append(pos)
            #poscar.write_file(os.path.join(fin_dir,'POSCAR'))
            poscar.write_file('POSCAR-'+str(vac_dir)+str(".vasp"))
            #incar.write_file(os.path.join(fin_dir,'INCAR'))
            #if ptcr_flag:
            #    potcar.write_file(os.path.join(fin_dir,'POTCAR'))
            #kpoints.write_file(os.path.join(fin_dir,'KPOINTS'))

            # Antisite generation at all vacancy sites
            struct_species = scs[0].types_of_specie
            for specie in set(struct_species)-set([vac_site_specie]):
                subspecie_symbol = specie.symbol
                anti_struct = sc.copy()
                anti_struct.append(specie, vac_site.frac_coords)
                poscar = mpvis.get_poscar(anti_struct)
                incar = mpvis.get_incar(anti_struct)
                incar.update(def_vasp_incar_param)
                as_dir ='antisite_{}_mult-{}_sitespecie-{}_subspecie-{}'.format(
                        str(i), site_mult, vac_symbol, subspecie_symbol)
                fin_dir = os.path.join(interdir,as_dir)
                #try:
                #    os.makedirs(fin_dir)
                #except:
                #    pass
                poscar.comment=str(as_dir)+str('@')+str('cellmax')+str(cellmax)
                pos=poscar
                #pos=poscar.structure
                def_str.append(pos)
                #poscar.write_file(os.path.join(fin_dir,'POSCAR'))
                poscar.write_file('POSCAR-'+str(as_dir)+str(".vasp"))
                #incar.write_file(os.path.join(fin_dir,'INCAR'))
                #if ptcr_flag:
                #        potcar.write_file(os.path.join(fin_dir,'POTCAR'))
                #kpoints.write_file(os.path.join(fin_dir,'KPOINTS'))
    #try:
    #    struct.make_supercell(sc_scale) 
    #    intl = Interstitial(struct, val, rad)
    #    cell_arr=sc_scale
    #    for el in struct.composition.elements:
    #        scs = intl.make_supercells_with_defects(1,el)
    #        #scs = intl.make_supercells_with_defects(cell_arr,el)
    #        for i in range(1,len(scs)):
    #           pos=Poscar(scs[i])
    #           pos.comment=str('intl_')+str('cellmax')+str(cellmax)+str('@')+str(intl.get_defectsite_coordination_number(i-1))+str('Element')+str(el)
    #           def_str.append(pos)
    #           pos.write_file('POSCAR-'+str('intl_')+str('cellmax')+str(cellmax)+str('@')+str(intl.get_defectsite_coordination_number(i-1))+str('Element')+str(el))

    #except:
    #      pass

    return def_str
def pmg_surfer(mpid='',vacuum=15,mat=None,max_index=1,min_slab_size=15):
    if mat == None:
        with MPRester() as mp:
               mat = mp.get_structure_by_material_id(mpid)
        if mpid == '':
          print ('Provide structure')

    sg_mat = SpacegroupAnalyzer(mat)
    mat_cvn = sg_mat.get_conventional_standard_structure()
    mat_cvn.sort()
    indices = get_symmetrically_distinct_miller_indices(mat_cvn, max_index)
    #ase_atoms = AseAtomsAdaptor().get_atoms(mat_cvn)

    structures=[]
    pos=Poscar(mat_cvn)
    try:
       pos.comment=str('sbulk')+str('@')+str('vac')+str(vacuum)+str('@')+str('size')+str(min_slab_size)
    except:
       pass
    structures.append(pos)
    mat_cvn.to(fmt='poscar',filename=str('POSCAR-')+str('cvn')+str('.vasp'))
    for i in indices:
        slab=SlabGenerator(initial_structure = mat_cvn, miller_index=i, min_slab_size= min_slab_size, min_vacuum_size=vacuum , lll_reduce=False, center_slab=True, primitive=False).get_slab()
        normal_slab = slab.get_orthogonal_c_slab()
        slab_pymatgen = Poscar(normal_slab).structure
        #ase_slab.center(vacuum=vacuum, axis=2)
        #slab_pymatgen = AseAtomsAdaptor().get_structure(ase_slab)
        xy_size=min_slab_size
        dim1=int((float(xy_size)/float( max(abs(slab_pymatgen.lattice.matrix[0])))))+1
        dim2=int(float(xy_size)/float( max(abs(slab_pymatgen.lattice.matrix[1]))))+1
        slab_pymatgen.make_supercell([dim1,dim2,1])
        slab_pymatgen.sort()
        surf_name='_'.join(map(str,i))
        pos=Poscar(slab_pymatgen)
        try:
           pos.comment=str("Surf-")+str(surf_name)+str('@')+str('vac')+str(vacuum)+str('@')+str('size')+str(min_slab_size)
        except:
           pass
        pos.write_file(filename=str('POSCAR-')+str("Surf-")+str(surf_name)+str('.vasp'))
        structures.append(pos)

    return structures
def surfer(mpid='',vacuum=15,layers=2,mat=None,max_index=1):
    if mat == None:
        with MPRester() as mp:
               mat = mp.get_structure_by_material_id(mpid)
        if mpid == '':
          print ('Provide structure')

    sg_mat = SpacegroupAnalyzer(mat)
    mat_cvn = sg_mat.get_conventional_standard_structure()
    mat_cvn.sort()
    indices = get_symmetrically_distinct_miller_indices(mat_cvn, max_index)
    ase_atoms = AseAtomsAdaptor().get_atoms(mat_cvn)

    structures=[]
    pos=Poscar(mat_cvn)
    try:
       pos.comment=str('sbulk')+str('@')+str('vac')+str(vacuum)+str('@')+str('layers')+str(layers)
    except:
       pass
    structures.append(pos)
    mat_cvn.to(fmt='poscar',filename=str('POSCAR-')+str('cvn')+str('.vasp'))
    for i in indices:
        ase_slab = surface(ase_atoms, i, layers)
        ase_slab.center(vacuum=vacuum, axis=2)
        slab_pymatgen = AseAtomsAdaptor().get_structure(ase_slab)
        slab_pymatgen.sort()
        surf_name='_'.join(map(str,i))
        pos=Poscar(slab_pymatgen)
        try:
           pos.comment=str("Surf-")+str(surf_name)+str('@')+str('vac')+str(vacuum)+str('@')+str('layers')+str(layers)
        except:
           pass
        pos.write_file(filename=str('POSCAR-')+str("Surf-")+str(surf_name)+str('.vasp'))
        structures.append(pos)

    return structures


def vac_intl(cellmax=2,mpid='',struct=None):


    if struct ==None:
        with MPRester() as mp:
                struct = mp.get_structure_by_material_id(mpid)
        if mpid == '':
           print ("Provide structure")
    sg_mat = SpacegroupAnalyzer(struct)
    struct = sg_mat.get_conventional_standard_structure()
    def_str=[]
    count=0
    cell_arr=[cellmax,cellmax,cellmax]
    vac = Vacancy(struct, {}, {})
    for el in list(struct.symbol_set):
   
        scs = vac.make_supercells_with_defects(cell_arr,el)
        for i in range(len(scs)):
            if i==0:
               pos=Poscar(scs[i])
               pos.comment=str('bulk')+str('.')+str('cellmax')+str(cellmax)
               if count==0:
                   def_str.append(pos)
                   count=count+1
            else:
               pos=Poscar(scs[i])
               pos.comment=str('vac')+str('cellmax')+str(cellmax)+str('@')+str(vac.get_defectsite_multiplicity(i))+str('Element')+str(el)
               if pos not in def_str:
                   def_str.append(pos)
    struct_valrad_eval = ValenceIonicRadiusEvaluator(struct)
    val = struct_valrad_eval.valences
    rad = struct_valrad_eval.radii
    struct_val = val
    struct_rad = rad
    intl = Interstitial(struct, val, rad)

    for el in struct.composition.elements:
        scs = intl.make_supercells_with_defects(cell_arr,el)
        for i in range(1,len(scs)):
           pos=Poscar(scs[i])
           pos.comment=str('intl')+str('cellmax')+str(cellmax)+str('@')+str(intl.get_defectsite_coordination_number(i-1))+str('Element')+str(el)
           if pos not in def_str:
                   def_str.append(pos)
    print (len(def_str))

def main():
    pp=vac_antisite_def_struct_gen(cellmax=2,mpid='mp-134')
    #pp=vac_intl(cellmax=128,mpid='mp-134')
#    ss=surfer(mpid='mp-134')
#    print ss
#main()
#strt = Structure.from_file("POSCAR")
#vac_antisite_def_struct_gen(struct=strt)




from numpy import matrix
# Copyright (c) Kamal Choudhary
import time
import numpy as np
import glob
from pymatgen.core.structure import Structure
from pymatgen.io.vasp.inputs import Incar, Poscar
from pymatgen.core.surface import  Slab, SlabGenerator, generate_all_slabs,get_symmetrically_distinct_miller_indices
from ase.lattice.surface import surface
from pymatgen.ext.matproj import MPRester
#from pymatgen.matproj.rest import MPRester
import operator
from pymatgen.core.lattice import Lattice
from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
import numpy as np,time,json
import sys,os,subprocess,socket
from pymatgen.io.ase import AseAtomsAdaptor
from ase.calculators.lammpsrun import LAMMPS
import sys,zipfile
import fortranformat as fform
from pymatgen.core.structure import Structure
import os
from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
from pymatgen.core.periodic_table import Element
import sys,shutil
import urllib2
import urllib
from BeautifulSoup import BeautifulSoup
import json
MAPI_KEY = os.environ.get("MAPI_KEY", "")
from pymatgen.matproj.rest import MPRester
from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
from math import ceil
from collections import OrderedDict

import matplotlib
matplotlib.use('Agg')

from pymatgen.core.composition import Composition
from pymatgen.phasediagram.entries import PDEntry
#from pymatgen.phasediagram.pdmaker import PhaseDiagram
#from pymatgen.phasediagram.plotter import PDPlotter

import glob

###################MAIN PART############################################
for file in glob.glob("*.alloy"):
    try:
        folder1=str(os.getcwd())+str("/")+str(file)+str("_nist")
        if not os.path.exists(folder1):
           os.makedirs(str(folder1))
        cwd1=str(os.getcwd())
        print ("folder1=",folder1)
        ff=str(file)
        element_ff=[]
        f=open(ff,'r')
        os.chdir(str(folder1))
        list_el=[]
        lines=f.readlines()
        content=(lines[3]).split(" ")
    #content=(lines[3]).split("' '|\n|\r\n")
        for val in content:
        
            if val != '' and val !='\n' and val !='\r\n':
               list_el.append(val)
        for i in range(0,len(list_el)):
             if i!=0:
                 element_ff.append(list_el[i])    
#    print ff,' ',element_ff
        with MPRester(MAPI_KEY) as m:
             data = m.get_entries_in_chemsys(element_ff,inc_structure='final', property_data=["unit_cell_formula","material_id","icsd_id","spacegroup","energy_per_atom","formation_energy_per_atom","pretty_formula","band_gap","total_magnetization","e_above_hull"])
             if (len(element_ff)>1):
                 try:
                     entries = m.get_entries_in_chemsys(element_ff)
                     pd = PhaseDiagram(entries)
                     plotter = PDPlotter(pd, show_unstable=True)
                     image=str(ff)+str("_DFT")+str(".jpg")
                     plotter.write_image(image)
                 except:
                     pass
             structures=[]
             structures_cvn=[]
             icsd_arr=[]
             mp_arr=[]
             sg_arr=[]
             enp_arr=[]
             fenp_arr=[]
             pf_arr=[]
             ucf_arr=[]
             bg_arr=[]
             tm_arr=[]
             ehull_arr=[]
             for d in data:
                 x=d.data["material_id"]
                 sg=d.data["spacegroup"]
                 enp=d.data["energy_per_atom"]
                 fenp=d.data["formation_energy_per_atom"]
                 pf=d.data["pretty_formula"]
                 ucf=d.data["unit_cell_formula"]
                 bg=d.data["band_gap"]
                 tm=d.data["total_magnetization"]
                 ehull=d.data["e_above_hull"]
                 icsd=d.data["icsd_id"]
                 structure = m.get_structure_by_material_id(x)
                 structures.append(structure)
                 icsd_arr.append(icsd)
                 mp_arr.append(x)
                 sg_arr.append(sg)
                 enp_arr.append(enp)
                 fenp_arr.append(fenp)
                 pf_arr.append(pf)
                 bg_arr.append(bg)
                 tm_arr.append(tm)
                 ucf_arr.append(ucf)
                 ehull_arr.append(ehull)
              

                 comment=str("bulk@")+str(x)
                 folder2=str(os.getcwd())+str("/")+str(comment)+str("_fold")
                 if not os.path.exists(folder2):
                     os.makedirs(str(folder2))
                 print ("folder2=",folder2)
                 cwd2=str(os.getcwd())
                 os.chdir(str(folder2))

                 p=Poscar(structure)
                 p.comment=comment
                 p.write_file("POSCAR")
                 poscar_file=str(os.getcwd())+str("/POSCAR")

                 pair_coeff=str(cwd1)+str("/")+str(file)
                 #pair_coeff=str('/data/knc6/JARVIS-FF-NEW/ALLOY')+str("/")+str(file)
                 parameters = {'exec':'mpirun /cluster/bin/lmp_ctcms-14439-knc6-2 <in.elastic >out','pair_style':'eam/alloy','pair_coeff':pair_coeff,'atom_style': 'charge' ,'control_file':'/users/knc6/inelast.mod'}
                 main_file=open("setup.py","w")
                 line=str("from jarvis.lammps.NEW_LAMMPS import main_func")+'\n'
                 main_file.write(line)
                 line=str("from pymatgen.io.vasp.inputs import  Poscar")+'\n'
                 main_file.write(line)
                 #line=str("try:")+'\n'
                 #main_file.write(line)
                 line=str("p=Poscar.from_file(")+str('"')+str(poscar_file)+str('"')+str(")")+'\n'
                 main_file.write(line)
                 line=str("main_func(mat=p")+str(",")+str("parameters=")+str(parameters)+str(")")+'\n'
                 main_file.write(line)
                 main_file.close()
                 cmd=str("nohup python setup.py &")
                 os.system(cmd)
                 #line=str("except:")+'\n'
                 #main_file.write(line)
                 #line=str("     pass")+'\n'
                 #main_file.write(line)
                 #try:
                 #    p=Poscar.from_file(poscar_file)
                 #    main_func(mat=p,parameters=parameters)
                 #except:
                 #     pass
                 os.chdir(cwd2)#=str(os.getcwd())
        os.chdir(cwd1)#=str(os.getcwd())
    except:
       pass



import glob
import os

from setuptools import setup, find_packages

JARVIS_DIR = os.path.dirname(os.path.abspath(__file__))

setup(
    name="jarvis-tools",
    version="2018.10.20",
    install_requires=["custodian>=1.0.4","ase==3.13.0"],
    extras_require={"babel": ["openbabel", "pybel"],
                    "remote": ["fabric"],
                    "doc": ["sphinx>=1.3.1", "sphinx-rtd-theme>=0.1.8"]
                    },
    author="Kamal Choudhary",
    author_email = "kamal.choudhary@nist.gov",
    description=(
        "High throughput computation with density functional theory, molecular dynamics and machine learning"),
    license="MIT",
    url="https://github.com/usnistgov/jarvis",
    packages=find_packages(),
    long_description=open(
        os.path.join(os.path.dirname(__file__), 'README.md')).read(),
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering",
    ],
    #scripts=glob.glob(os.path.join(JARVIS_DIR,  "*"))
)

